@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -44,15 +44,11 @@ my_bool server_has_i_s(DBC FAR *dbc)
   */
   return is_minimum_version(dbc->mysql.server_version, "5.0.2", 5);
 }
-
-ulong no_information_schema(STMT FAR *stmt)
-{
-  return stmt->dbc->flag & FLAG_NO_INFORMATION_SCHEMA;
-}
 /*
   @type    : internal
   @purpose : returns the next token
 */
+
 const char *my_next_token(const char *prev_token, 
                                 char **token, 
                                 char *data, 
@@ -96,9 +92,14 @@ create_fake_resultset(STMT *stmt, MYSQL_ROW rowval, size_t rowsize,
   stmt->result_array= (MYSQL_ROW)my_memdup((char *)rowval, rowsize, MYF(0));
   if (!(stmt->result && stmt->result_array))
   {
-    x_free(stmt->result);
-    x_free(stmt->result_array);
-
+    if (stmt->result)
+    {
+      my_free((char *)stmt->result, MYF(0));
+    }
+    if (stmt->result_array)
+    {
+      my_free((char *)stmt->result_array, MYF(0));
+    }
     set_mem_error(&stmt->dbc->mysql);
     return handle_connection_error(stmt);
   }
@@ -172,7 +173,7 @@ static MYSQL_RES *mysql_table_status_i_s(STMT        *stmt,
   }
   else
   {
-    to= strmov(to, "TABLE_SCHEMA = DATABASE() ");
+    to= strmov(to, "TABLE_SCHEMA = DATABASE()");
   }
 
   if (show_tables)
@@ -214,11 +215,8 @@ static MYSQL_RES *mysql_table_status_i_s(STMT        *stmt,
     to= strmov(to, "'");
   }
 
-  assert(to - buff < sizeof(buff));
-
   MYLOG_QUERY(stmt, buff);
-
-  if (mysql_real_query(mysql, buff, (unsigned long)(to - buff)))
+  if (mysql_query(mysql,buff))
     return NULL;
 
   return mysql_store_result(mysql);
@@ -247,7 +245,7 @@ MYSQL_RES *mysql_table_status(STMT        *stmt,
                               my_bool      show_tables,
                               my_bool      show_views)
 {
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
     return mysql_table_status_i_s(stmt, catalog, catalog_length,
                                   table, table_length, wildcard,
                                              show_tables, show_views);
@@ -270,7 +268,7 @@ int add_name_condition_oa_id(HSTMT hstmt, char ** pos, SQLCHAR * name,
   SQLUINTEGER metadata_id;
 
   /* this shouldn't be very expensive, so no need to complicate things with additional parameter etc */
-  SQLGetStmtAttr(hstmt, SQL_ATTR_METADATA_ID, (SQLPOINTER)&metadata_id, 0, NULL);
+  MySQLGetStmtAttr(hstmt, SQL_ATTR_METADATA_ID, (SQLPOINTER)&metadata_id, 0, NULL);
 
   /* we can't rely here that column was checked and is not null */
   if (name)
@@ -314,7 +312,7 @@ int add_name_condition_pv_id(HSTMT hstmt, char ** pos, SQLCHAR * name,
 {
   SQLUINTEGER metadata_id;
   /* this shouldn't be very expensive, so no need to complicate things with additional parameter etc */
-  SQLGetStmtAttr(hstmt, SQL_ATTR_METADATA_ID, (SQLPOINTER)&metadata_id, 0, NULL);
+  MySQLGetStmtAttr(hstmt, SQL_ATTR_METADATA_ID, (SQLPOINTER)&metadata_id, 0, NULL);
 
   /* we can't rely here that column was checked and is not null */
   if (name)
@@ -367,7 +365,7 @@ i_s_tables(SQLHSTMT hstmt,
 
 
 SQLRETURN SQL_API
-SQLTables(SQLHSTMT hstmt,
+MySQLTables(SQLHSTMT hstmt,
             SQLCHAR *catalog, SQLSMALLINT catalog_len,
             SQLCHAR *schema, SQLSMALLINT schema_len,
             SQLCHAR *table, SQLSMALLINT table_len,
@@ -387,7 +385,7 @@ SQLTables(SQLHSTMT hstmt,
   if (type_len == SQL_NTS)
     type_len= type ? (SQLSMALLINT)strlen((char *)type) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     return i_s_tables(hstmt, catalog, catalog_len, schema, schema_len,
                       table, table_len, type, type_len);
@@ -419,7 +417,7 @@ SQLColumns
   @param[in] cbColumn         Length of column pattern
 */
 SQLRETURN
-i_s_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
+i_s_columns(SQLHSTMT hstmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
             SQLCHAR *szSchema __attribute__((unused)),
             SQLSMALLINT cbSchema __attribute__((unused)),
             SQLCHAR *szTable, SQLSMALLINT cbTable,
@@ -427,7 +425,7 @@ i_s_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
 
 {
   /* The function is just a stub. We call non-I_S version of the function before implementing the I_S one */
-  return mysql_columns(stmt, szCatalog, cbCatalog,szSchema, cbSchema,
+  return mysql_columns(hstmt, szCatalog, cbCatalog,szSchema, cbSchema,
                        szTable, cbTable, szColumn, cbColumn);
 }
 
@@ -446,7 +444,7 @@ i_s_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
   @param[in] cbColumn         Length of column pattern
 */
 SQLRETURN SQL_API
-SQLColumns(SQLHSTMT hstmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
+MySQLColumns(SQLHSTMT hstmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
              SQLCHAR *szSchema __attribute__((unused)),
              SQLSMALLINT cbSchema __attribute__((unused)),
              SQLCHAR *szTable, SQLSMALLINT cbTable,
@@ -465,14 +463,14 @@ SQLColumns(SQLHSTMT hstmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
   if (cbTable == SQL_NTS)
     cbTable= szTable != NULL ? (SQLSMALLINT)strlen((char *)szTable) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
-    return i_s_columns(stmt, szCatalog, cbCatalog,szSchema, cbSchema,
+    return i_s_columns(hstmt, szCatalog, cbCatalog,szSchema, cbSchema,
                          szTable, cbTable, szColumn, cbColumn);
   }
   else
   {
-    return mysql_columns(stmt, szCatalog, cbCatalog,szSchema, cbSchema,
+    return mysql_columns(hstmt, szCatalog, cbCatalog,szSchema, cbSchema,
                          szTable, cbTable, szColumn, cbColumn);
   }
 }
@@ -513,13 +511,13 @@ i_s_statistics(SQLHSTMT hstmt,
 */
 
 SQLRETURN SQL_API
-SQLStatistics(SQLHSTMT hstmt,
-              SQLCHAR *catalog, SQLSMALLINT catalog_len,
-              SQLCHAR *schema __attribute__((unused)),
-              SQLSMALLINT schema_len __attribute__((unused)),
-              SQLCHAR *table, SQLSMALLINT table_len,
-              SQLUSMALLINT fUnique,
-              SQLUSMALLINT fAccuracy __attribute__((unused)))
+MySQLStatistics(SQLHSTMT hstmt,
+                SQLCHAR *catalog, SQLSMALLINT catalog_len,
+                SQLCHAR *schema __attribute__((unused)),
+                SQLSMALLINT schema_len __attribute__((unused)),
+                SQLCHAR *table, SQLSMALLINT table_len,
+                SQLUSMALLINT fUnique,
+                SQLUSMALLINT fAccuracy __attribute__((unused)))
 {
   STMT *stmt= (STMT *)hstmt;
 
@@ -532,7 +530,7 @@ SQLStatistics(SQLHSTMT hstmt,
   if (table_len == SQL_NTS)
     table_len= table ? (SQLSMALLINT)strlen((char *)table) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     return i_s_statistics(hstmt, catalog, catalog_len, schema, schema_len,
                           table, table_len, fUnique, fAccuracy);
@@ -584,7 +582,7 @@ SQLRETURN i_s_list_table_priv(SQLHSTMT    hstmt,
 
   assert(pos - buff < sizeof(buff));
 
-  if( !SQL_SUCCEEDED(rc= my_SQLPrepare(hstmt, (SQLCHAR *)buff, (SQLINTEGER)(pos - buff))))
+  if( !SQL_SUCCEEDED(rc= MySQLPrepare(hstmt, (SQLCHAR *)buff, (SQLINTEGER)(pos - buff), FALSE)))
     return rc;
 
   return my_SQLExecute(stmt);
@@ -598,7 +596,7 @@ SQLRETURN i_s_list_table_priv(SQLHSTMT    hstmt,
              set on the specified statement.
 */
 SQLRETURN SQL_API
-SQLTablePrivileges(SQLHSTMT hstmt,
+MySQLTablePrivileges(SQLHSTMT hstmt,
                      SQLCHAR *catalog, SQLSMALLINT catalog_len,
                      SQLCHAR *schema __attribute__((unused)),
                      SQLSMALLINT schema_len __attribute__((unused)),
@@ -614,7 +612,7 @@ SQLTablePrivileges(SQLHSTMT hstmt,
     if (table_len == SQL_NTS)
       table_len= table ? (SQLSMALLINT)strlen((char *)table) : 0;
 
-    if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+    if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
     {
       /* Since mysql is also the name of the system db, using here i_s prefix to
          distinct functions */
@@ -672,7 +670,7 @@ static SQLRETURN i_s_list_column_priv(HSTMT *     hstmt,
 
   assert(pos - buff < sizeof(buff));
 
-  if( !SQL_SUCCEEDED(rc= my_SQLPrepare(hstmt, (SQLCHAR *)buff, SQL_NTS)))
+  if( !SQL_SUCCEEDED(rc= MySQLPrepare(hstmt, (SQLCHAR *)buff, SQL_NTS, FALSE)))
     return rc;
 
   return my_SQLExecute(stmt);
@@ -680,7 +678,7 @@ static SQLRETURN i_s_list_column_priv(HSTMT *     hstmt,
 
 
 SQLRETURN SQL_API
-SQLColumnPrivileges(SQLHSTMT hstmt,
+MySQLColumnPrivileges(SQLHSTMT hstmt,
                       SQLCHAR *catalog, SQLSMALLINT catalog_len,
                       SQLCHAR *schema __attribute__((unused)),
                       SQLSMALLINT schema_len __attribute__((unused)),
@@ -699,7 +697,7 @@ SQLColumnPrivileges(SQLHSTMT hstmt,
   if (column_len == SQL_NTS)
     column_len= column ? (SQLSMALLINT)strlen((char *)column) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     /* Since mysql is also the name of the system db, using here i_s prefix to
     distinct functions */
@@ -746,7 +744,7 @@ i_s_special_columns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
 */
 
 SQLRETURN SQL_API
-SQLSpecialColumns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
+MySQLSpecialColumns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
                     SQLCHAR *szTableQualifier, SQLSMALLINT cbTableQualifier,
                     SQLCHAR *szTableOwner __attribute__((unused)),
                     SQLSMALLINT cbTableOwner __attribute__((unused)),
@@ -764,7 +762,7 @@ SQLSpecialColumns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
   if (cbTableName == SQL_NTS)
     cbTableName= szTableName ? (SQLSMALLINT)strlen((char *)szTableName) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     return i_s_special_columns(hstmt, fColType, szTableQualifier,
                                  cbTableQualifier, szTableOwner, cbTableOwner,
@@ -806,7 +804,7 @@ i_s_primary_keys(SQLHSTMT hstmt,
 */
 
 SQLRETURN SQL_API
-SQLPrimaryKeys(SQLHSTMT hstmt,
+MySQLPrimaryKeys(SQLHSTMT hstmt,
                  SQLCHAR *catalog, SQLSMALLINT catalog_len,
                  SQLCHAR *schema __attribute__((unused)),
                  SQLSMALLINT schema_len __attribute__((unused)),
@@ -822,7 +820,7 @@ SQLPrimaryKeys(SQLHSTMT hstmt,
   if (table_len == SQL_NTS)
     table_len= table ? (SQLSMALLINT)strlen((char *)table) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     return i_s_primary_keys(hstmt, catalog, catalog_len, schema, schema_len,
                               table, table_len);
@@ -971,7 +969,7 @@ SQLRETURN i_s_foreign_keys(SQLHSTMT hstmt,
 
   assert(buff - query < sizeof(query));
 
-  rc= my_SQLPrepare(hstmt, (SQLCHAR *)query, (SQLINTEGER)(buff - query));
+  rc= MySQLPrepare(hstmt, (SQLCHAR *)query, (SQLINTEGER)(buff - query), FALSE);
 
   if (!SQL_SUCCEEDED(rc))
     return rc;
@@ -1007,7 +1005,7 @@ SQLRETURN i_s_foreign_keys(SQLHSTMT hstmt,
   @since ODBC 1.0
 */
 SQLRETURN SQL_API
-SQLForeignKeys(SQLHSTMT hstmt,
+MySQLForeignKeys(SQLHSTMT hstmt,
                  SQLCHAR *szPkCatalogName __attribute__((unused)),
                  SQLSMALLINT cbPkCatalogName __attribute__((unused)),
                  SQLCHAR *szPkSchemaName __attribute__((unused)),
@@ -1035,7 +1033,7 @@ SQLForeignKeys(SQLHSTMT hstmt,
     if (cbFkTableName == SQL_NTS)
       cbFkTableName= szFkTableName != NULL ? (SQLSMALLINT)strlen((char *)szFkTableName) : 0;
 
-    if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+    if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
     {
       return i_s_foreign_keys(hstmt, szPkCatalogName, cbPkCatalogName, szPkSchemaName,
                               cbPkSchemaName, szPkTableName, cbPkTableName, szFkCatalogName,
@@ -1072,7 +1070,7 @@ SQLProcedures and SQLProcedureColumns
   @param[in] cbProcName       Length of procedure name
 */
 SQLRETURN SQL_API
-SQLProcedures(SQLHSTMT hstmt,
+MySQLProcedures(SQLHSTMT hstmt,
                 SQLCHAR *szCatalogName, SQLSMALLINT cbCatalogName,
                 SQLCHAR *szSchemaName __attribute__((unused)),
                 SQLSMALLINT cbSchemaName __attribute__((unused)),
@@ -1091,7 +1089,7 @@ SQLProcedures(SQLHSTMT hstmt,
       We use the server to generate a fake result with no rows, but
       reasonable column information.
     */
-    if ((rc= my_SQLPrepare(hstmt, (SQLCHAR *)"SELECT "
+    if ((rc= MySQLPrepare(hstmt, (SQLCHAR *)"SELECT "
                           "'' AS PROCEDURE_CAT,"
                           "'' AS PROCEDURE_SCHEM,"
                           "'' AS PROCEDURE_NAME,"
@@ -1100,7 +1098,7 @@ SQLProcedures(SQLHSTMT hstmt,
                           "NULL AS NUM_RESULT_SETS,"
                           "'' AS REMARKS,"
                           "0 AS PROCEDURE_TYPE "
-                          "FROM DUAL WHERE 1=0", SQL_NTS)))
+                          "FROM DUAL WHERE 1=0", SQL_NTS, FALSE)))
       return rc;
 
     return my_SQLExecute(hstmt);
@@ -1112,7 +1110,7 @@ SQLProcedures(SQLHSTMT hstmt,
     behavior, but seems useful.)
   */
   if (szCatalogName && szProcName)
-    rc= my_SQLPrepare(hstmt, (SQLCHAR *)
+    rc= MySQLPrepare(hstmt, (SQLCHAR *)
                      "SELECT ROUTINE_SCHEMA AS PROCEDURE_CAT,"
                      "NULL AS PROCEDURE_SCHEM,"
                      "ROUTINE_NAME AS PROCEDURE_NAME,"
@@ -1124,9 +1122,9 @@ SQLProcedures(SQLHSTMT hstmt,
                        "IF(ROUTINE_TYPE= 'PROCEDURE', 1, 0)) AS PROCEDURE_TYPE"
                      "  FROM INFORMATION_SCHEMA.ROUTINES"
                      " WHERE ROUTINE_NAME LIKE ? AND ROUTINE_SCHEMA = ?",
-                     SQL_NTS);
+                     SQL_NTS, FALSE);
   else if (szProcName)
-    rc= my_SQLPrepare(hstmt, (SQLCHAR *)
+    rc= MySQLPrepare(hstmt, (SQLCHAR *)
                      "SELECT ROUTINE_SCHEMA AS PROCEDURE_CAT,"
                      "NULL AS PROCEDURE_SCHEM,"
                      "ROUTINE_NAME AS PROCEDURE_NAME,"
@@ -1139,9 +1137,9 @@ SQLProcedures(SQLHSTMT hstmt,
                      "  FROM INFORMATION_SCHEMA.ROUTINES"
                      " WHERE ROUTINE_NAME LIKE ?"
                      " AND ROUTINE_SCHEMA = DATABASE()",
-                     SQL_NTS);
+                     SQL_NTS, FALSE);
   else
-    rc= my_SQLPrepare(hstmt, (SQLCHAR *)
+    rc= MySQLPrepare(hstmt, (SQLCHAR *)
                      "SELECT ROUTINE_SCHEMA AS PROCEDURE_CAT,"
                      "NULL AS PROCEDURE_SCHEM,"
                      "ROUTINE_NAME AS PROCEDURE_NAME,"
@@ -1153,7 +1151,7 @@ SQLProcedures(SQLHSTMT hstmt,
                        "IF(ROUTINE_TYPE= 'PROCEDURE', 1, 0)) AS PROCEDURE_TYPE"
                      " FROM INFORMATION_SCHEMA.ROUTINES"
                      " WHERE ROUTINE_SCHEMA = DATABASE()",
-                     SQL_NTS);
+                     SQL_NTS, FALSE);
   if (!SQL_SUCCEEDED(rc))
     return rc;
 
@@ -1218,7 +1216,7 @@ i_s_procedure_columns(SQLHSTMT hstmt,
 */
 
 SQLRETURN SQL_API
-SQLProcedureColumns(SQLHSTMT hstmt,
+MySQLProcedureColumns(SQLHSTMT hstmt,
                     SQLCHAR *szCatalogName, SQLSMALLINT cbCatalogName,
                     SQLCHAR *szSchemaName __attribute__((unused)),
                     SQLSMALLINT cbSchemaName __attribute__((unused)),
@@ -1239,7 +1237,7 @@ SQLProcedureColumns(SQLHSTMT hstmt,
   if (cbColumnName == SQL_NTS)
     cbColumnName= szColumnName ? (SQLSMALLINT)strlen((char *)szColumnName) : 0;
 
-  if (server_has_i_s(stmt->dbc) && !no_information_schema(stmt))
+  if (server_has_i_s(stmt->dbc) && !stmt->dbc->ds->no_information_schema)
   {
     return i_s_procedure_columns(hstmt, szCatalogName, cbCatalogName, szSchemaName,
                                  cbSchemaName, szProcName, cbProcName, szColumnName,
