@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -167,6 +167,7 @@ MYSQL_FIELD SQLCOLUMNS_fields[]=
 
 const uint SQLCOLUMNS_FIELDS= array_elements(SQLCOLUMNS_values);
 
+
 /**
   Get the list of columns in a table matching a wildcard.
 
@@ -178,60 +179,109 @@ const uint SQLCOLUMNS_FIELDS= array_elements(SQLCOLUMNS_values);
   @param[in] szColumn         Pattern of column names to match
   @param[in] cbColumn         Length of column pattern
 
-  @return Result of mysql_list_fields, or NULL if there is an error
+  @return Result of SHOW COLUMNS FROM, or NULL if there is an error
 */
-static MYSQL_RES *
-mysql_list_dbcolumns(STMT *stmt,
-                     SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
-                     SQLCHAR *szTable, SQLSMALLINT cbTable,
-                     SQLCHAR *szColumn, SQLSMALLINT cbColumn)
+static
+MYSQL_RES *mysql_list_dbcolumns(STMT *stmt,
+                                SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
+                                SQLCHAR *szTable, SQLSMALLINT cbTable,
+                                SQLCHAR *szColumn, SQLSMALLINT cbColumn)
 {
   DBC *dbc= stmt->dbc;
   MYSQL *mysql= &dbc->mysql;
   MYSQL_RES *result;
-  char buff[256], column_buff[64*2+1];
+  MYSQL_ROW row;
 
-  /* If a catalog was specified, we have to change working catalog
-     to be able to use mysql_list_fields. */
+  /* If a catalog was specified, we have to do it the hard way. */
   if (cbCatalog)
   {
-    if (reget_current_catalog(dbc))
-      return NULL;
+    char buff[255];
+    char *select, *to;
+    size_t select_len;
+    unsigned long *lengths;
+
+    /* Get a list of column names that match our criteria. */
+    to= strmov(buff, "SHOW COLUMNS FROM `");
+    if (cbCatalog)
+    {
+      to+= myodbc_escape_string(mysql, to, (ulong)(sizeof(buff) - (to - buff)),
+                                (char *)szCatalog, cbCatalog, 1);
+      to= strmov(to, "`.`");
+    }
+
+    to+= myodbc_escape_string(mysql, to, (ulong)(sizeof(buff) - (to - buff)),
+                              (char *)szTable, cbTable, 1);
+    to= strmov(to, "`");
+
+    if (cbColumn)
+    {
+      to= strmov(to, " LIKE '");
+      to+= mysql_real_escape_string(mysql, to, (char *)szColumn, cbColumn);
+      to= strmov(to, "'");
+    }
+
+    MYLOG_QUERY(stmt, buff);
 
-    /* reget_current_catalog locks and release mutex, so locking
-       here again */
     pthread_mutex_lock(&dbc->lock);
-
-    strncpy(buff, szCatalog, cbCatalog);
-    buff[cbCatalog]= '\0';
-
-    if (mysql_select_db(mysql, buff))
+    if (mysql_query(mysql,buff) ||
+        !(result= mysql_store_result(mysql)))
     {
       pthread_mutex_unlock(&dbc->lock);
       return NULL;
     }
-  }
-  else
-    pthread_mutex_lock(&dbc->lock);
+    pthread_mutex_unlock(&dbc->lock);
 
-  strncpy(buff, szTable, cbTable);
-  buff[cbTable]= '\0';
-  strncpy(column_buff, szColumn, cbColumn);
-  column_buff[cbColumn]= '\0';
-
-  result= mysql_list_fields(mysql, buff, column_buff);
-
-  /* If before this call no database were selected - we cannot revert that */
-  if (cbCatalog && dbc->database)
-  {
-    if (mysql_select_db( mysql, dbc->database))
+    /* Build a SELECT ... LIMIT 0 to get the field metadata. */
+    select_len = (ulong)result->row_count * (NAME_LEN + 1) + NAME_LEN * 2;
+    if (!(select= (char *)my_malloc(select_len, MYF(0))))
     {
-      /* Well, probably have to return error here */
-      mysql_free_result(result);
+      set_mem_error(mysql);
+      return NULL;
+    }
+
+    to= strxmov(select, "SELECT ", NullS);
+    while ((row= mysql_fetch_row(result)))
+    {
+      to= strmov(to, "`");
+      lengths= mysql_fetch_lengths(result);
+      to+= myodbc_escape_string(mysql, to, (ulong)(select_len - (to - select)),
+                                (char *)row[0], lengths[0], 1);
+      to= strmov(to, "`,");
+    }
+    *(--to)= '\0';
+
+    to= strmov(to, " FROM `");
+    if (cbCatalog)
+    {
+      to+= myodbc_escape_string(mysql, to, (ulong)(select_len - (to - select)),
+                                (char *)szCatalog, cbCatalog, 1);
+      to= strmov(to, "`.`");
+    }
+
+    to+= myodbc_escape_string(mysql, to, (ulong)(select_len - (to - select)),
+                              (char *)szTable, cbTable, 1);
+    to= strmov(to, "` LIMIT 0");
+
+    mysql_free_result(result);
+
+    MYLOG_QUERY(stmt, select);
+
+    pthread_mutex_lock(&dbc->lock);
+    if (mysql_query(mysql, select))
+    {
+      my_free(select, MYF(0));
       pthread_mutex_unlock(&dbc->lock);
       return NULL;
     }
+    result= mysql_store_result(&dbc->mysql);
+    pthread_mutex_unlock(&dbc->lock);
+    my_free(select, MYF(0));
+
+    return result;
   }
+
+  pthread_mutex_lock(&dbc->lock);
+  result= mysql_list_fields(mysql, (char *)szTable, (char *)szColumn);
   pthread_mutex_unlock(&dbc->lock);
 
   return result;
@@ -252,24 +302,23 @@ mysql_list_dbcolumns(STMT *stmt,
   @param[in] cbColumn         Length of column pattern
 */
 SQLRETURN
-mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
+mysql_columns(SQLHSTMT hstmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
              SQLCHAR *szSchema __attribute__((unused)),
              SQLSMALLINT cbSchema __attribute__((unused)),
              SQLCHAR *szTable, SQLSMALLINT cbTable,
              SQLCHAR *szColumn, SQLSMALLINT cbColumn)
 
 {
+  STMT *stmt= (STMT *)hstmt;
   MYSQL_RES *res;
   MEM_ROOT *alloc;
   MYSQL_ROW table_row;
   unsigned long rows= 0, next_row= 0, *lengths;
   char *db= NULL;
-  BOOL is_access= FALSE;
 
   /* Get the list of tables that match szCatalog and szTable */
   pthread_mutex_lock(&stmt->dbc->lock);
-  res= mysql_table_status(stmt, szCatalog, cbCatalog, szTable, cbTable, TRUE,
-                          TRUE, TRUE);
+  res= mysql_table_status(stmt, szCatalog, cbCatalog, szTable, cbTable, TRUE, TRUE, TRUE);
 
   if (!res && mysql_errno(&stmt->dbc->mysql))
   {
@@ -284,15 +333,10 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
   }
   pthread_mutex_unlock(&stmt->dbc->lock);
 
-#ifdef _WIN32
-  if (GetModuleHandle("msaccess.exe") != NULL)
-    is_access= TRUE;
-#endif
-
   stmt->result= res;
   alloc= &res->field_alloc;
 
-  if (!(stmt->dbc->flag & FLAG_NO_CATALOG))
+  if (!stmt->dbc->ds->no_catalog)
     db= strmake_root(alloc, (char *)szCatalog, cbCatalog);
 
   while ((table_row= mysql_fetch_row(res)))
@@ -357,7 +401,7 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
       }
 
       /* COLUMN_SIZE */
-      fill_column_size_buff(buff, stmt, field, FALSE);
+      fill_column_size_buff(buff, stmt, field);
       row[6]= strdup_root(alloc, buff);
 
       /* BUFFER_LENGTH */
@@ -366,13 +410,9 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
 
       if (is_char_sql_type(type) || is_wchar_sql_type(type) ||
           is_binary_sql_type(type))
-      {
         row[15]= strdup_root(alloc, buff); /* CHAR_OCTET_LENGTH */
-      }
       else
-      {
         row[15]= NULL;                     /* CHAR_OCTET_LENGTH */
-      }
 
       {
         SQLSMALLINT digits= get_decimal_digits(stmt, field);
@@ -398,23 +438,9 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
       if ((field->flags & NOT_NULL_FLAG) && !(field->flags & TIMESTAMP_FLAG) &&
           !(field->flags & AUTO_INCREMENT_FLAG))
       {
-        /* Bug#31067. Access seems to try to put NULL value when not null field
-           is cleared. And that contradicts with its knowledge of that the field
-           is not nullable, and it yields an error. Here is a little trick for
-           such case - we don't tell Access the whole truth we know, and
-           return for such field SQL_NULLABLE_UNKNOWN instead*/
-        if (is_access)
-        {
-          sprintf(buff, "%d", SQL_NULLABLE_UNKNOWN);
-          row[10]= strdup_root(alloc, buff); /* NULLABLE */
-          row[17]= strdup_root(alloc, "NO");/* IS_NULLABLE */
-        }
-        else
-        {
-          sprintf(buff, "%d", SQL_NO_NULLS);
-          row[10]= strdup_root(alloc, buff); /* NULLABLE */
-          row[17]= strdup_root(alloc, "NO"); /* IS_NULLABLE */
-        }
+        sprintf(buff, "%d", SQL_NO_NULLS);
+        row[10]= strdup_root(alloc, buff); /* NULLABLE */
+        row[17]= strdup_root(alloc, "NO"); /* IS_NULLABLE */
       }
       else
       {
@@ -426,17 +452,17 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
       row[11]= ""; /* REMARKS */
 
       /*
-        The default value of the column. The value in this column should be
-        interpreted as a string if it is enclosed in quotation marks.
+         The default value of the column. The value in this column should be
+         interpreted as a string if it is enclosed in quotation marks.
 
-        if NULL was specified as the default value, then this column is the
-        word NULL, not enclosed in quotation marks. If the default value
-        cannot be represented without truncation, then this column contains
-        TRUNCATED, with no enclosing single quotation marks. If no default
-        value was specified, then this column is NULL.
+         if NULL was specified as the default value, then this column is the
+         word NULL, not enclosed in quotation marks. If the default value
+         cannot be represented without truncation, then this column contains
+         TRUNCATED, with no enclosing single quotation marks. If no default
+         value was specified, then this column is NULL.
 
-        The value of COLUMN_DEF can be used in generating a new column
-        definition, except when it contains the value TRUNCATED
+         The value of COLUMN_DEF can be used in generating a new column
+         definition, except when it contains the value TRUNCATED
       */
       if (!field->def)
         row[12]= NullS; /* COLUMN_DEF */
@@ -444,20 +470,14 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
       {
         if (field->type == MYSQL_TYPE_TIMESTAMP &&
             !strcmp(field->def,"0000-00-00 00:00:00"))
-        {
           row[12]= NullS; /* COLUMN_DEF */
-        }
         else
         {
           char *def= alloc_root(alloc, strlen(field->def) + 3);
           if (is_numeric_mysql_type(field))
-          {
             sprintf(def, "%s", field->def);
-          }
           else
-          {
             sprintf(def, "'%s'", field->def);
-          }
           row[12]= def; /* COLUMN_DEF */
         }
       }
@@ -926,10 +946,10 @@ SQLRETURN mysql_foreign_keys(SQLHSTMT hstmt,
 
   pthread_mutex_lock(&stmt->dbc->lock);
 
-  stmt->result= mysql_table_status(stmt,
+  stmt->result= mysql_table_status_show(stmt,
                                    szFkCatalogName, cbFkCatalogName,
                                    szFkTableName, cbFkTableName,
-                                   FALSE, TRUE, FALSE);
+                                   FALSE);
 
   if (!stmt->result)
   {
@@ -1022,9 +1042,7 @@ SQLRETURN mysql_foreign_keys(SQLHSTMT hstmt,
               /* FKTABLE_CAT */
               data[4]= (szFkCatalogName ?
                         strdup_root(alloc, (char *)szFkCatalogName) :
-                        (stmt->dbc->database ?
-                          strdup_root(alloc, stmt->dbc->database) :
-                          "null"));
+                        strdup_root(alloc, stmt->dbc->database));
               data[5]= NULL;                         /* FKTABLE_SCHEM */
               data[6]= row[0];                       /* FKTABLE_TABLE */
 
@@ -1076,7 +1094,7 @@ SQLRETURN mysql_foreign_keys(SQLHSTMT hstmt,
                                            SQLFORE_KEYS_FIELDS *
                                            row_count,
                                            MYF(0));
-  x_free((char *)tempdata);
+  my_free((char *)tempdata, MYF(0));
 
   if (!stmt->result_array)
   {
@@ -1305,14 +1323,12 @@ static void free_procedurecolumn_res(int total_records, LIST *params)
          && (j != mypcREMARKS)
          && (j != mypcCOLUMN_DEF)
          && (j != mypcIS_NULLABLE))
-        {
-          x_free(((char**)cur_params->data)[j]);
-        }
+          my_free(((char**)cur_params->data)[j], MYF(0));
       }
       /* cleanup the list */
       params= list_delete_forward(params);
-      x_free(cur_params->data);
-      x_free(cur_params);
+      my_free(cur_params->data, MYF(0));
+      my_free(cur_params, MYF(0));
     }
   }
 }
@@ -1659,7 +1675,7 @@ clean_exit:
   }
 
   dynstr_free(&dynQuery);
-  x_free(params_r);
+  my_free(params_r, MYF(0));
 
   return nReturn;
 }
@@ -1767,7 +1783,7 @@ mysql_special_columns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
             row[3]= strdup_root(alloc,buff);
             sprintf(buff,"%d",type);
             row[2]= strdup_root(alloc,buff);
-            fill_column_size_buff(buff, stmt, field, FALSE);
+            fill_column_size_buff(buff, stmt, field);
             row[4]= strdup_root(alloc,buff);
             sprintf(buff, "%ld", get_transfer_octet_length(stmt, field));
             row[5]= strdup_root(alloc,buff);
@@ -1843,7 +1859,7 @@ mysql_special_columns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
         row[3]= strdup_root(alloc,buff);
         sprintf(buff,"%d",type);
         row[2]= strdup_root(alloc,buff);
-        fill_column_size_buff(buff, stmt, field, FALSE);
+        fill_column_size_buff(buff, stmt, field);
         row[4]= strdup_root(alloc,buff);
         sprintf(buff,"%ld", get_transfer_octet_length(stmt, field));
         row[5]= strdup_root(alloc,buff);
@@ -1940,7 +1956,7 @@ mysql_statistics(SQLHSTMT hstmt,
       return handle_connection_error(stmt);
     }
 
-    if (stmt->dbc->flag & FLAG_NO_CATALOG)
+    if (stmt->dbc->ds->no_catalog)
       stmt->array[0]= "";
     else
       stmt->array[0]= strmake_root(&stmt->result->field_alloc,
@@ -2150,23 +2166,11 @@ mysql_tables(SQLHSTMT hstmt,
 
       data= stmt->result_array;
 
-      if (!(stmt->dbc->flag & FLAG_NO_CATALOG))
+      if (!stmt->dbc->ds->no_catalog)
       {
-        if (!catalog)
-        {
-          if (!reget_current_catalog(stmt->dbc))
-          {
-            const char *dbname= stmt->dbc->database ? stmt->dbc->database
-                                                    : "null";
-            db= strmake_root(&stmt->result->field_alloc,
-                             dbname, strlen(dbname));
-          }
-          else
-          {
-            /* error was set in reget_current_catalog */
-            return SQL_ERROR;
-          }
-        }
+        if (!catalog && !reget_current_catalog(stmt->dbc))
+          db= strmake_root(&stmt->result->field_alloc,
+                           stmt->dbc->database, strlen(stmt->dbc->database));
         else
           db= strmake_root(&stmt->result->field_alloc,
                            (char *)catalog, catalog_len);
@@ -2179,7 +2183,7 @@ mysql_tables(SQLHSTMT hstmt,
         my_bool view;
 
         /* TODO Check this condition is really needed here */
-        if (no_information_schema(stmt)
+        if (stmt->dbc->ds->no_information_schema
           || !server_has_i_s(stmt->dbc))
           type_index= comment_index= (stmt->result->field_count == 18) ? 17 : 15;
 
