@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2001, 2010, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -27,15 +27,6 @@
   @brief Error handling functions.
 */
 
-/***************************************************************************
- * The following ODBC APIs are implemented in this file:		   *
- *									   *
- *   SQLGetDiagField	 (ISO 92)					   *
- *   SQLGetDiagRec	 (ISO 92)					   *
- *   SQLError		 (ODBC, Deprecated)				   *
- *									   *
- ****************************************************************************/
-
 #include "driver.h"
 #include "mysqld_error.h"
 #include "errmsg.h"
@@ -74,12 +65,15 @@ static MYODBC3_ERR_STR myodbc3_errors[]=
   {"HY002","Invalid column number", SQL_ERROR},
   {"HY003","Invalid application buffer type", SQL_ERROR},
   {"HY004","Invalid SQL data type", SQL_ERROR},
+  {"HY007","Associated statement is not prepared", SQL_ERROR},
   {"HY009","Invalid use of null pointer", SQL_ERROR},
   {"HY010","Function sequence error", SQL_ERROR},
   {"HY011","Attribute can not be set now", SQL_ERROR},
   {"HY012","Invalid transaction operation code", SQL_ERROR},
   {"HY013","Memory management error", SQL_ERROR},
   {"HY015","No cursor name available", SQL_ERROR},
+  {"HY016","Cannot modify an implementation row descriptor", SQL_ERROR},
+  {"HY017","Invalid use of an automatically allocated descriptor handle", SQL_ERROR},
   {"HY024","Invalid attribute value", SQL_ERROR},
   {"HY090","Invalid string or buffer length", SQL_ERROR},
   {"HY091","Invalid descriptor field identifier", SQL_ERROR},
@@ -116,7 +110,7 @@ void myodbc_sqlstate2_init(void)
       in set_error/set_conn_error
     */
     uint  i;
-    for ( i= MYERR_S1000; i <= MYERR_S1C00; i++ )
+    for ( i= MYERR_S1000; i <= MYERR_S1C00; ++i )
     {
         myodbc3_errors[i].sqlstate[0]= 'S';
         myodbc3_errors[i].sqlstate[1]= '1';
@@ -140,7 +134,7 @@ void myodbc_sqlstate3_init(void)
 {
     uint i;
 
-    for ( i= MYERR_S1000; i <= MYERR_S1C00; i++ )
+    for ( i= MYERR_S1000; i <= MYERR_S1C00; ++i )
     {
         myodbc3_errors[i].sqlstate[0]= 'H';
         myodbc3_errors[i].sqlstate[1]= 'Y';
@@ -157,7 +151,7 @@ void myodbc_sqlstate3_init(void)
 
 /*
   @type    : myodbc3 internal
-  @purpose : copies error from one handdle to other
+  @purpose : copies error from one handle to other
 */
 
 SQLRETURN copy_stmt_error(STMT FAR *dst,STMT FAR *src)
@@ -205,6 +199,25 @@ SQLRETURN set_stmt_error( STMT FAR *    stmt,
 }
 
 
+/*
+  @type    : myodbc3 internal
+  @purpose : sets the descriptor level errors
+*/
+
+SQLRETURN set_desc_error(DESC *        desc,
+                         char *        state,
+                         const char *  message,
+                         uint          errcode)
+{
+  strmov(desc->error.sqlstate, state);
+  strxmov(desc->error.message, desc->stmt->dbc->st_error_prefix,
+          message, NullS);
+  desc->error.native_error = errcode;
+
+  return SQL_ERROR;
+}
+
+
 /*
   @type    : myodbc3 internal
   @purpose : translates SQL error to ODBC error
@@ -347,8 +360,6 @@ SQLRETURN handle_connection_error(STMT *stmt)
 {
   unsigned int err= mysql_errno(&stmt->dbc->mysql);
   switch (err) {
-  case 0:  /* no error */
-    return SQL_SUCCESS;
   case CR_SERVER_GONE_ERROR:
   case CR_SERVER_LOST:
     return set_stmt_error(stmt, "08S01", mysql_error(&stmt->dbc->mysql), err);
@@ -374,98 +385,91 @@ SQLRETURN set_handle_error(SQLSMALLINT HandleType, SQLHANDLE handle,
 {
     switch ( HandleType )
     {
-        
         case SQL_HANDLE_ENV:
             return copy_error(&((ENV *)handle)->error,errid,errtext,
                               errcode,MYODBC3_ERROR_PREFIX);
-
         case SQL_HANDLE_DBC:
             return copy_error(&((DBC *)handle)->error,errid,errtext,
                               errcode,MYODBC3_ERROR_PREFIX);
-
-        default:
+        case SQL_HANDLE_STMT:
             return copy_error(&((STMT *)handle)->error,errid,errtext,
                               errcode,((STMT *)handle)->dbc->st_error_prefix);
+        case SQL_HANDLE_DESC:
+            return copy_error(&((DESC *)handle)->error,errid,errtext,errcode,
+                              ((DESC *)handle)->stmt->dbc->st_error_prefix);
+        default:
+            return SQL_INVALID_HANDLE;
     }
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : returns the current values of multiple fields of a diagnostic
-  record that contains error, warning, and status information.
-  Unlike SQLGetDiagField, which returns one diagnostic field
-  per call, SQLGetDiagRec returns several commonly used fields
-  of a diagnostic record, including the SQLSTATE, the native
-  error code, and the diagnostic message text
+/**
 */
-
-SQLRETURN my_SQLGetDiagRec(SQLSMALLINT HandleType,
-                           SQLHANDLE Handle,
-                           SQLSMALLINT RecNumber,
-                           SQLCHAR *Sqlstate,
-                           SQLINTEGER  *NativeErrorPtr,
-                           SQLCHAR *MessageText,
-                           SQLSMALLINT BufferLength,
-                           SQLSMALLINT *TextLengthPtr)
+SQLRETURN SQL_API
+MySQLGetDiagRec(SQLSMALLINT handle_type, SQLHANDLE handle, SQLSMALLINT record,
+                SQLCHAR **sqlstate, SQLINTEGER *native, SQLCHAR **message)
 {
-    char *errmsg,tmp_state[6];
-    SQLSMALLINT tmp_size;
-    SQLINTEGER  tmp_error;
+  MYERROR *error;
+  SQLINTEGER tmp_native;
 
-    if ( !TextLengthPtr )
-        TextLengthPtr= &tmp_size;
+  if (!native)
+    native= &tmp_native;
 
-    if ( !Sqlstate )
-        Sqlstate= (SQLCHAR*) tmp_state;
+  if (!handle || record <= 0)
+    return SQL_ERROR;
 
-    if ( !NativeErrorPtr )
-        NativeErrorPtr= &tmp_error;
+  /*
+    Currently we are not supporting error list, so
+    if RecNumber > 1, return no data found
+  */
+  if (record > 1)
+    return SQL_NO_DATA_FOUND;
 
-    if ( RecNumber <= 0 || BufferLength < 0 || !Handle )
-        return SQL_ERROR;
+  if (handle_type == SQL_HANDLE_STMT)
+    error= &((STMT *)handle)->error;
+  else if (handle_type == SQL_HANDLE_DBC)
+    error= &((DBC *)handle)->error;
+  else if (handle_type == SQL_HANDLE_ENV)
+    error= &((ENV *)handle)->error;
+  else if (handle_type == SQL_HANDLE_DESC)
+    error= &((DESC *)handle)->error;
+  else
+    return SQL_INVALID_HANDLE;
 
-    /*
-      Currently we are not supporting error list, so
-      if RecNumber > 1, return no data found
-    */
+  if (!error->message || !error->message[0])
+  {
+    *message= (SQLCHAR *)"";
+    *sqlstate= (SQLCHAR *)"00000";
+    *native= 0;
+    return SQL_NO_DATA_FOUND;
+  }
 
-    if ( RecNumber > 1 )
-        return SQL_NO_DATA_FOUND;
+  *message= (SQLCHAR *)error->message;
+  *sqlstate= (SQLCHAR *)error->sqlstate;
+  *native= error->native_error;
 
-    switch ( HandleType )
-    {
-        case SQL_HANDLE_STMT:
-            errmsg= ((STMT FAR*) Handle)->error.message;
-            strmov((char*) Sqlstate,((STMT FAR*) Handle)->error.sqlstate);
-            *NativeErrorPtr= ((STMT FAR*) Handle)->error.native_error;
-            break;
+  return SQL_SUCCESS;
+}
 
-        case SQL_HANDLE_DBC:
-            errmsg= ((DBC FAR*) Handle)->error.message;
-            strmov((char*) Sqlstate,((DBC FAR*) Handle)->error.sqlstate);
-            *NativeErrorPtr= ((DBC FAR*) Handle)->error.native_error;
 
-            break;
+my_bool is_odbc3_subclass(char *sqlstate)
+{
+  char *states[]= { "01S00", "01S01", "01S02", "01S06", "01S07", "07S01",
+    "08S01", "21S01", "21S02", "25S01", "25S02", "25S03", "42S01", "42S02",
+    "42S11", "42S12", "42S21", "42S22", "HY095", "HY097", "HY098", "HY099",
+    "HY100", "HY101", "HY105", "HY107", "HY109", "HY110", "HY111", "HYT00",
+    "HYT01", "IM001", "IM002", "IM003", "IM004", "IM005", "IM006", "IM007",
+    "IM008", "IM010", "IM011", "IM012"};
+  size_t i;
 
-        case SQL_HANDLE_ENV:
-            errmsg= ((ENV FAR*) Handle)->error.message;
-            strmov((char*) Sqlstate,((ENV FAR*) Handle)->error.sqlstate);
-            *NativeErrorPtr= ((ENV FAR*) Handle)->error.native_error;
-            break;
+  if (!sqlstate)
+    return FALSE;
 
-        default:
-            return SQL_INVALID_HANDLE;
-    }
-    if ( !errmsg || !errmsg[0] )
-    {
-        *TextLengthPtr= 0;
-        strmov((char*) Sqlstate, "00000");
-        return SQL_NO_DATA_FOUND;
-    }
+  for (i= 0; i < sizeof(states) / sizeof(states[0]); ++i)
+    if (memcmp(states[i], sqlstate, 5) == 0)
+      return TRUE;
 
-    return copy_str_data(HandleType, Handle, MessageText, BufferLength,
-                         TextLengthPtr, (char *)errmsg);
+  return FALSE;
 }
 
 
@@ -475,248 +479,184 @@ SQLRETURN my_SQLGetDiagRec(SQLSMALLINT HandleType,
   diagnostic data structure (associated with a specified handle)
   that contains error, warning, and status information
 */
-
-SQLRETURN SQL_API SQLGetDiagField(SQLSMALLINT HandleType,
-                                  SQLHANDLE   Handle,
-                                  SQLSMALLINT RecNumber,
-                                  SQLSMALLINT DiagIdentifier,
-                                  SQLPOINTER  DiagInfoPtr,
-                                  SQLSMALLINT BufferLength,
-                                  SQLSMALLINT *StringLengthPtr)
+SQLRETURN SQL_API
+MySQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle, SQLSMALLINT record,
+                  SQLSMALLINT identifier, SQLCHAR **char_value,
+                  SQLPOINTER num_value)
 {
-    SQLRETURN   error= SQL_SUCCESS;
-    SQLPOINTER  szDiagInfo= NULL;
-    SQLSMALLINT tmp_size;
+  SQLLEN num;
 
-    if ( !StringLengthPtr )
-        StringLengthPtr= &tmp_size;
+  /* Handle may not be these types, but this saves lots of casts below. */
+  STMT *stmt= (STMT *)handle;
+  DBC *dbc= (DBC *)handle;
+  DESC *desc= (DESC *)handle;
+  MYERROR *error;
 
-    if ( !DiagInfoPtr )
-        DiagInfoPtr= szDiagInfo;
+  if (!num_value)
+    num_value= &num;
 
-    if ( !Handle ||
-         !(HandleType == SQL_HANDLE_STMT ||
-           HandleType == SQL_HANDLE_DBC ||
-           HandleType == SQL_HANDLE_ENV) )
+  if (!handle)
+    return SQL_ERROR;
+
+  if (handle_type == SQL_HANDLE_DESC)
+    error= &desc->error;
+  else if (handle_type == SQL_HANDLE_STMT)
+    error= &stmt->error;
+  else if (handle_type == SQL_HANDLE_DBC)
+    error= &dbc->error;
+  else if (handle_type == SQL_HANDLE_ENV)
+    error= &((ENV *)handle)->error;
+  else
+    return SQL_ERROR;
+
+  if (record > 1)
+    return SQL_NO_DATA_FOUND;
+
+  switch (identifier)
+  {
+  /*  Header fields */
+  case SQL_DIAG_CURSOR_ROW_COUNT:
+    if (handle_type != SQL_HANDLE_STMT)
+      return SQL_ERROR;
+    if (!stmt->result)
+      *(SQLLEN *)num_value= 0;
+    else
+      *(SQLLEN *)num_value= (SQLLEN) mysql_num_rows(stmt->result);
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_DYNAMIC_FUNCTION:
+    if (handle_type != SQL_HANDLE_STMT)
+      return SQL_ERROR;
+    *char_value= (SQLCHAR *)"";
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_DYNAMIC_FUNCTION_CODE:
+    if (handle_type != SQL_HANDLE_STMT)
+      return SQL_ERROR;
+    *(SQLINTEGER *)num_value= 0;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_NUMBER:
+    *(SQLINTEGER *)num_value= 1;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_RETURNCODE:
+    *(SQLRETURN *)num_value= error->retcode;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_ROW_COUNT:
+    if (handle_type != SQL_HANDLE_STMT)
+      return SQL_ERROR;
+    if (!stmt->result)
+      *(SQLLEN *)num_value= 0;
+    else
+      *(SQLLEN *)num_value= (SQLLEN)stmt->affected_rows;
+    return SQL_SUCCESS;
+
+  /* Record fields */
+  case SQL_DIAG_CLASS_ORIGIN:
+    {
+      char *sqlstate;
+      if (record <= 0)
         return SQL_ERROR;
+      sqlstate= error->sqlstate;
 
-    if ( RecNumber > 1 )
-        return SQL_NO_DATA_FOUND;
-
-    switch ( DiagIdentifier )
-    {
-
-        /* DIAG HEADER FIELDS SECTION */
-        case SQL_DIAG_DYNAMIC_FUNCTION:
-            if ( HandleType != SQL_HANDLE_STMT )
-                return SQL_ERROR;
-
-            error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                 StringLengthPtr, "");
-            break;
-
-        case SQL_DIAG_DYNAMIC_FUNCTION_CODE:
-            *(SQLINTEGER *) DiagInfoPtr= 0;
-            break;
-
-        case SQL_DIAG_ROW_NUMBER:
-            *(SQLINTEGER *) DiagInfoPtr= SQL_ROW_NUMBER_UNKNOWN;
-            break;
-
-        case SQL_DIAG_NUMBER:
-            *(SQLINTEGER *) DiagInfoPtr= 1;
-            break;
-
-        case SQL_DIAG_RETURNCODE:
-            if ( HandleType == SQL_HANDLE_STMT )
-                *(SQLRETURN *) DiagInfoPtr= ((STMT FAR*) Handle)->error.retcode;
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                *(SQLRETURN *) DiagInfoPtr= ((DBC FAR*) Handle)->error.retcode;
-
-            else
-                *(SQLRETURN *) DiagInfoPtr= ((ENV FAR*) Handle)->error.retcode;
-            break;
-
-        case SQL_DIAG_CURSOR_ROW_COUNT:/* at present, return total rows in rs */
-            if ( HandleType != SQL_HANDLE_STMT )
-                return SQL_ERROR;
-
-            if ( !((STMT FAR*) Handle)->result )
-                *(SQLINTEGER *) DiagInfoPtr= 0;
-
-            else
-                *(SQLINTEGER *) DiagInfoPtr= (SQLINTEGER)
-                                             mysql_num_rows(((STMT FAR*)  Handle)->result);
-            break;
-
-        case SQL_DIAG_ROW_COUNT:
-            if ( HandleType != SQL_HANDLE_STMT )
-                return SQL_ERROR;
-
-            *(SQLINTEGER *) DiagInfoPtr= (SQLINTEGER)
-                                         ((STMT FAR*)  Handle)->affected_rows;
-            break;
-
-            /* DIAG RECORD FIELDS SECTION */
-
-        case SQL_DIAG_CLASS_ORIGIN:
-        case SQL_DIAG_SUBCLASS_ORIGIN:
-            error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                 StringLengthPtr, "ISO 9075");
-            break;
-
-        case SQL_DIAG_COLUMN_NUMBER:
-            *(SQLINTEGER *) DiagInfoPtr= SQL_COLUMN_NUMBER_UNKNOWN;
-            break;
-
-        case SQL_DIAG_CONNECTION_NAME:
-            /*
-              When the connection fails, ODBC DM calls this function, so don't
-              return dbc->dsn as the connection name instead return empty string
-            */
-
-            if ( HandleType == SQL_HANDLE_STMT )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,((STMT FAR*) Handle)->dbc->dsn ?
-                                     ((STMT FAR*) Handle)->dbc->dsn:"");
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,((DBC FAR*) Handle)->dsn ?
-                                     ((DBC FAR*) Handle)->dsn:"");
-
-            else
-            {
-                *(SQLCHAR *) DiagInfoPtr= 0;
-                *StringLengthPtr= 0;
-            }
-            break;
-
-        case SQL_DIAG_MESSAGE_TEXT:
-            if ( HandleType == SQL_HANDLE_STMT )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,
-                                     ((STMT FAR*) Handle)->error.message);
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,((DBC FAR*) Handle)->error.message);
-            else
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,((ENV FAR*) Handle)->error.message);
-            break;
-
-        case SQL_DIAG_NATIVE:
-            if ( HandleType == SQL_HANDLE_STMT )
-                *(SQLINTEGER *) DiagInfoPtr= ((STMT FAR*) Handle)->error.native_error;
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                *(SQLINTEGER *) DiagInfoPtr= ((DBC FAR*) Handle)->error.native_error;
-
-            else
-                *(SQLINTEGER *) DiagInfoPtr= ((ENV FAR*) Handle)->error.native_error;
-            break;
-
-        case SQL_DIAG_SERVER_NAME:
-            if ( HandleType == SQL_HANDLE_STMT )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,
-                                     ((STMT FAR*) Handle)->dbc->server ?
-                                     ((STMT FAR*) Handle)->dbc->server : "");
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,((DBC FAR*) Handle)->server ?
-                                     ((DBC FAR*) Handle)->server : "");
-            else
-            {
-                *(SQLCHAR *) DiagInfoPtr= 0;
-                *StringLengthPtr= 0;
-            }
-            break;
-
-        case SQL_DIAG_SQLSTATE:
-            if ( HandleType == SQL_HANDLE_STMT )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,
-                                     ((STMT FAR*) Handle)->error.sqlstate);
-
-            else if ( HandleType == SQL_HANDLE_DBC )
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,
-                                     ((DBC FAR*) Handle)->error.sqlstate);
-
-            else
-                error= copy_str_data(HandleType, Handle, DiagInfoPtr, BufferLength,
-                                     StringLengthPtr,
-                                     ((ENV FAR*) Handle)->error.sqlstate);
-            break;
-
-        default:
-            return SQL_ERROR;
+      if (sqlstate && sqlstate[0] == 'I' && sqlstate[1] == 'M')
+        *char_value= (SQLCHAR *)"ODBC 3.0";
+      else
+        *char_value= (SQLCHAR *)"ISO 9075";
     }
-    return error;
-}
-
-
-/*
-  @type    : ODBC 3.0 API
-  @purpose : returns the current diagnostic record information
-*/
-
-SQLRETURN SQL_API SQLGetDiagRec(SQLSMALLINT HandleType,
-                                SQLHANDLE   Handle,
-                                SQLSMALLINT RecNumber,
-                                SQLCHAR     *Sqlstate,
-                                SQLINTEGER  *NativeErrorPtr,
-                                SQLCHAR     *MessageText,
-                                SQLSMALLINT BufferLength,
-                                SQLSMALLINT *TextLengthPtr)
-{
-    return my_SQLGetDiagRec(HandleType, Handle, RecNumber, Sqlstate,
-                            NativeErrorPtr, MessageText, BufferLength,
-                            TextLengthPtr);
-}
-
-
-/*
-  @type    : ODBC 1.0 API - depricated
-  @purpose : returns error or status information
-*/
-
-SQLRETURN SQL_API SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt,
-                           SQLCHAR FAR    *szSqlState,
-                           SQLINTEGER FAR *pfNativeError,
-                           SQLCHAR FAR    *szErrorMsg,
-                           SQLSMALLINT    cbErrorMsgMax,
-                           SQLSMALLINT FAR *pcbErrorMsg)
-{
-    SQLRETURN error= SQL_INVALID_HANDLE;
-
-    if ( hstmt )
-    {
-        error= my_SQLGetDiagRec(SQL_HANDLE_STMT,hstmt,1,szSqlState,
-                                pfNativeError, szErrorMsg,
-                                cbErrorMsgMax,pcbErrorMsg);
-        if ( error == SQL_SUCCESS )
-            CLEAR_STMT_ERROR(hstmt);
-    }
-    else if ( hdbc )
-    {
-        error= my_SQLGetDiagRec(SQL_HANDLE_DBC,hdbc,1,szSqlState,
-                                pfNativeError, szErrorMsg,
-                                cbErrorMsgMax,pcbErrorMsg);
-        if ( error == SQL_SUCCESS )
-            CLEAR_DBC_ERROR(hdbc);
-    }
-    else if ( henv )
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_COLUMN_NUMBER:
+    if (record <= 0)
+      return SQL_ERROR;
+    *(SQLINTEGER *)num_value= SQL_COLUMN_NUMBER_UNKNOWN;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_CONNECTION_NAME:
+  {
+    DataSource *ds;
+    if (record <= 0)
+      return SQL_ERROR;
+
+    if (handle_type == SQL_HANDLE_DESC)
+      ds= desc->stmt->dbc->ds;
+    else if (handle_type == SQL_HANDLE_STMT)
+      ds= stmt->dbc->ds;
+    else if (handle_type == SQL_HANDLE_DBC)
+      ds= dbc->ds;
+    else
+      *char_value= (SQLCHAR *)"";
+    
+    if (ds)
+      *char_value= (SQLCHAR *)ds->name8;
+    return SQL_SUCCESS;
+  }
+
+  case SQL_DIAG_MESSAGE_TEXT:
+    if (record <= 0)
+      return SQL_ERROR;
+    *char_value= (SQLCHAR *)error->message;
+    if (!*char_value)
+      *char_value= (SQLCHAR *)"";
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_NATIVE:
+    *(SQLINTEGER *)num_value= error->native_error;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_ROW_NUMBER:
+    if (record <= 0)
+      return SQL_ERROR;
+    *(SQLLEN *)num_value= SQL_ROW_NUMBER_UNKNOWN;
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_SERVER_NAME:
+  {
+    DataSource *ds;
+    if (record <= 0)
+      return SQL_ERROR;
+    if (handle_type == SQL_HANDLE_DESC)
+      ds= desc->stmt->dbc->ds;
+    else if (handle_type == SQL_HANDLE_STMT)
+      ds= stmt->dbc->ds;
+    else if (handle_type == SQL_HANDLE_DBC)
+      ds= dbc->ds;
+    else
+      *char_value= (SQLCHAR *)"";
+
+    if (ds)
+      *char_value= (SQLCHAR *)ds->server8;
+    
+    return SQL_SUCCESS;
+  }
+
+  case SQL_DIAG_SQLSTATE:
+    if (record <= 0)
+      return SQL_ERROR;
+    *char_value= (SQLCHAR *)error->sqlstate;
+    if (!*char_value)
+      *char_value= (SQLCHAR *)"";
+    return SQL_SUCCESS;
+
+  case SQL_DIAG_SUBCLASS_ORIGIN:
+    if (record <= 0)
+      return SQL_ERROR;
     {
-        error= my_SQLGetDiagRec(SQL_HANDLE_ENV,henv,1,szSqlState,
-                                pfNativeError, szErrorMsg,
-                                cbErrorMsgMax,pcbErrorMsg);
-        if ( error == SQL_SUCCESS )
-            CLEAR_ENV_ERROR(henv);
+      char *sqlstate;
+      if (record <= 0)
+        return SQL_ERROR;
+      sqlstate= error->sqlstate;
+
+      if (is_odbc3_subclass(sqlstate))
+        *char_value= (SQLCHAR *)"ODBC 3.0";
+      else
+        *char_value= (SQLCHAR *)"ISO 9075";
     }
-    return error;
+    return SQL_SUCCESS;
+
+  default:
+    return SQL_ERROR;
+  }
 }
