@@ -27,22 +27,6 @@
   @brief Functions for handling handle attributes and options.
 */
 
-/***************************************************************************
- * The following ODBC APIs are implemented in this file:		   *
- *									   *
- *   SQLSetEnvAttr	 (ISO 92)					   *
- *   SQLGetEnvAttr	 (ISO 92)					   *
- *   SQLSetConnectAttr	 (ISO 92)					   *
- *   SQLGetConnectAttr	 (ISO 92)					   *
- *   SQLSetStmtAttr	 (ISO 92)					   *
- *   SQLGetStmtAttr	 (ISO 92)					   *
- *   SQLSetConnectOption (ODBC, Deprecated)				   *
- *   SQLGetConnectOption (ODBC, Deprecated)				   *
- *   SQLSetStmtOption	 (ODBC, Deprecated)				   *
- *   SQLGetStmtOption	 (ODBC, Deprecated)				   *
- *									   *
- ****************************************************************************/
-
 #include "driver.h"
 #include "errmsg.h"
 
@@ -74,14 +58,14 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
             break;
 
         case SQL_ATTR_CURSOR_TYPE:
-            if (((STMT FAR*)Handle)->dbc->flag & FLAG_FORWARD_CURSOR)
+            if (((STMT FAR*)Handle)->dbc->ds->force_use_of_forward_only_cursors)
             {
                 options->cursor_type= SQL_CURSOR_FORWARD_ONLY;
                 if (ValuePtr != (SQLPOINTER)SQL_CURSOR_FORWARD_ONLY)
                     return set_handle_error(HandleType,Handle,MYERR_01S02,
                                             "Forcing the use of forward-only cursor)",0);
             }
-            else if (((STMT FAR*)Handle)->dbc->flag & FLAG_DYNAMIC_CURSOR)
+            else if (((STMT FAR*)Handle)->dbc->ds->dynamic_cursor)
             {
                 if (ValuePtr != (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN)
                     options->cursor_type= (SQLUINTEGER)(SQLULEN)ValuePtr;
@@ -125,38 +109,27 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
         case SQL_ATTR_RETRIEVE_DATA:
             break;
 
-        case SQL_ATTR_ROW_BIND_TYPE:
-            options->bind_type= (SQLUINTEGER)(SQLULEN)ValuePtr;
-            break;
-
         case SQL_ATTR_SIMULATE_CURSOR:
             if (ValuePtr != (SQLPOINTER) SQL_SC_TRY_UNIQUE)
                 return set_handle_error(HandleType,Handle,MYERR_01S02,
                                         "Option value changed to default cursor simulation",0);
             break;
 
-        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-            options->bind_offset= (SQLLEN *)ValuePtr;
-            break;
-
         case 1226:/* MS SQL Server Extension */
         case 1227:
         case 1228:
             break;
 
+        case SQL_ATTR_FETCH_BOOKMARK_PTR:
+        case SQL_ATTR_USE_BOOKMARKS:
+            return set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0);
+
         case SQL_ATTR_QUERY_TIMEOUT:
         case SQL_ATTR_KEYSET_SIZE:
         case SQL_ATTR_CONCURRENCY:
         case SQL_ATTR_NOSCAN:
-        case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-            break;
-
-        case SQL_ATTR_FETCH_BOOKMARK_PTR:
-        case SQL_ATTR_USE_BOOKMARKS:
-            return set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0);
-            break;
-
         default:
+            /* ignored */
             break;
     }
     return SQL_SUCCESS;
@@ -210,10 +183,6 @@ get_constmt_attr(SQLSMALLINT  HandleType,
             *((SQLUINTEGER *) ValuePtr)= SQL_RD_DEFAULT;
             break;
 
-        case SQL_ATTR_ROW_BIND_TYPE:
-            *((SQLUINTEGER *) ValuePtr)= options->bind_type;
-            break;
-
         case SQL_ATTR_SIMULATE_CURSOR:
             *((SQLUINTEGER *) ValuePtr)= SQL_SC_TRY_UNIQUE;
             break;
@@ -226,21 +195,10 @@ get_constmt_attr(SQLSMALLINT  HandleType,
             *((SQLUINTEGER *) ValuePtr)= 0L;
             break;
 
-        case SQL_ROWSET_SIZE:
-            *(SQLUINTEGER *)ValuePtr= options->rows_in_set;
-            break;
-
         case SQL_NOSCAN:
             *((SQLUINTEGER *) ValuePtr)= SQL_NOSCAN_ON;
             break;
 
-        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-            *((SQLLEN **) ValuePtr)= options->bind_offset;
-            break;
-
-        case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-            return SQL_SUCCESS_WITH_INFO;
-
         case SQL_ATTR_FETCH_BOOKMARK_PTR:
         case SQL_ATTR_USE_BOOKMARKS:
             return set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0);
@@ -248,9 +206,8 @@ get_constmt_attr(SQLSMALLINT  HandleType,
         case 1226:/* MS SQL Server Extension */
         case 1227:
         case 1228:
-            break;
-
         default:
+            /* ignored */
             break;
     }
     return SQL_SUCCESS;
@@ -262,11 +219,9 @@ get_constmt_attr(SQLSMALLINT  HandleType,
   @purpose : sets the connection attributes
 */
 
-static SQLRETURN
-set_con_attr(SQLHDBC    hdbc,
-             SQLINTEGER Attribute,
-             SQLPOINTER ValuePtr,
-             SQLINTEGER StringLengthPtr)
+SQLRETURN SQL_API
+MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
+                    SQLPOINTER ValuePtr, SQLINTEGER StringLengthPtr)
 {
     DBC FAR *dbc= (DBC FAR*) hdbc;
 
@@ -283,7 +238,7 @@ set_con_attr(SQLHDBC    hdbc,
                     dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
                     return SQL_SUCCESS;
                 }
-                if (!(trans_supported(dbc)) || (dbc->flag & FLAG_NO_TRANSACTIONS))
+                if (!(trans_supported(dbc)) || dbc->ds->disable_transactions)
                     return set_conn_error(dbc,MYERR_S1C00,
                                           "Transactions are not enabled", 4000);
 
@@ -346,7 +301,7 @@ set_con_attr(SQLHDBC    hdbc,
                         return SQL_ERROR;
                     }
                 }
-                x_free(dbc->database);
+                my_free(dbc->database,MYF(0));
                 dbc->database= my_strdup(db,MYF(MY_WME));
                 pthread_mutex_unlock(&dbc->lock);
             }
@@ -354,7 +309,7 @@ set_con_attr(SQLHDBC    hdbc,
 
 
         case SQL_ATTR_ODBC_CURSORS:
-            if ((dbc->flag & FLAG_FORWARD_CURSOR) &&
+            if (dbc->ds->force_use_of_forward_only_cursors &&
                 ValuePtr != (SQLPOINTER) SQL_CUR_USE_ODBC)
                 return set_conn_error(hdbc,MYERR_01S02,
                                       "Forcing the Driver Manager to use ODBC cursor library",0);
@@ -418,7 +373,7 @@ set_con_attr(SQLHDBC    hdbc,
             /*
               3.x driver doesn't support any statement attributes
               at connection level, but to make sure all 2.x apps
-              works fine...lets support it..nothing to loose..
+              works fine...lets support it..nothing to lose..
             */
         default:
             return set_constmt_attr(2, dbc, &dbc->stmt_options, Attribute,
@@ -428,173 +383,129 @@ set_con_attr(SQLHDBC    hdbc,
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : returns the connection attribute values
+/**
+ Retrieve connection attribute values.
+
+ @param[in]  hdbc
+ @param[in]  attrib
+ @param[out] char_attr
+ @param[out] num_attr
 */
-
-static SQLRETURN get_con_attr(SQLHDBC    hdbc,
-                              SQLINTEGER Attribute,
-                              SQLPOINTER ValuePtr,
-                              SQLINTEGER BufferLength,
-                              SQLINTEGER *StringLengthPtr)
+SQLRETURN SQL_API
+MySQLGetConnectAttr(SQLHDBC hdbc, SQLINTEGER attrib, SQLCHAR **char_attr,
+                    SQLPOINTER num_attr)
 {
-    DBC FAR *dbc= (DBC FAR*) hdbc;
-    SQLRETURN result= SQL_SUCCESS;
-    SQLINTEGER len;
-    SQLPOINTER vparam= 0;
+  DBC *dbc= (DBC *)hdbc;
+  SQLRETURN result= SQL_SUCCESS;
 
-    if (!ValuePtr)
-        ValuePtr= vparam;
+  switch (attrib) {
+  case SQL_ATTR_ACCESS_MODE:
+    *((SQLUINTEGER *)num_attr)= SQL_MODE_READ_WRITE;
+    break;
 
-    if (!StringLengthPtr)
-        StringLengthPtr= &len;
+  case SQL_ATTR_AUTO_IPD:
+    *((SQLUINTEGER *)num_attr)= SQL_FALSE;
+    break;
 
-    switch (Attribute)
+  case SQL_ATTR_AUTOCOMMIT:
+    *((SQLUINTEGER *)num_attr)= (autocommit_on(dbc) ||
+                                 (!(trans_supported(dbc)) ?
+                                  SQL_AUTOCOMMIT_ON :
+                                  SQL_AUTOCOMMIT_OFF));
+    break;
+
+  case SQL_ATTR_CONNECTION_DEAD:
+    if (mysql_ping(&dbc->mysql) &&
+        (mysql_errno(&dbc->mysql) == CR_SERVER_LOST ||
+         mysql_errno(&dbc->mysql) == CR_SERVER_GONE_ERROR))
+      *((SQLUINTEGER *)num_attr)= SQL_CD_TRUE;
+    else
+      *((SQLUINTEGER *)num_attr)= SQL_CD_FALSE;
+    break;
+
+  case SQL_ATTR_CONNECTION_TIMEOUT:
+    /* We don't support this option, so it is always 0. */
+    *((SQLUINTEGER *)num_attr)= 0;
+    break;
+
+  case SQL_ATTR_CURRENT_CATALOG:
+    if (is_connected(dbc) && reget_current_catalog(dbc))
+      return set_handle_error(SQL_HANDLE_DBC, hdbc, MYERR_S1000,
+                              "Unable to get current catalog", 0);
+    else
+      *char_attr= (SQLCHAR *)dbc->database;
+    break;
+
+  case SQL_ATTR_LOGIN_TIMEOUT:
+    *((SQLUINTEGER *)num_attr)= dbc->login_timeout;
+    break;
+
+  case SQL_ATTR_ODBC_CURSORS:
+    if (dbc->ds->force_use_of_forward_only_cursors)
+      *((SQLUINTEGER *)num_attr)= SQL_CUR_USE_ODBC;
+    else
+      *((SQLUINTEGER *)num_attr)= SQL_CUR_USE_IF_NEEDED;
+    break;
+
+  case SQL_ATTR_PACKET_SIZE:
+    *((SQLUINTEGER *)num_attr)= dbc->mysql.net.max_packet;
+    break;
+
+  case SQL_ATTR_TXN_ISOLATION:
+    /*
+      If we don't know the isolation level already, we need to ask the
+      server.
+    */
+    if (!dbc->txn_isolation)
     {
-        case SQL_ATTR_ACCESS_MODE:
-            *((SQLUINTEGER *) ValuePtr)= SQL_MODE_READ_WRITE;
-            break;
+      /*
+        Unless we're not connected yet, then we just assume it will
+        be REPEATABLE READ, which is the server default.
+      */
+      if (!is_connected(dbc))
+      {
+        *((SQLINTEGER *)num_attr)= SQL_TRANSACTION_REPEATABLE_READ;
+        break;
+      }
 
-        case SQL_ATTR_AUTO_IPD:
-            *((SQLUINTEGER *) ValuePtr)= SQL_FALSE;
-            break;
+      if (odbc_stmt(dbc, "SELECT @@tx_isolation"))
+      {
+        return set_handle_error(SQL_HANDLE_DBC, hdbc, MYERR_S1000,
+                                "Failed to get isolation level", 0);
+      }
+      else
+      {
+        MYSQL_RES *res;
+        MYSQL_ROW  row;
 
-        case SQL_ATTR_AUTOCOMMIT:
-            *((SQLUINTEGER *)ValuePtr)= (autocommit_on(dbc) ||
-                                         !(trans_supported(dbc)) ?
-                                         SQL_AUTOCOMMIT_ON :
-                                         SQL_AUTOCOMMIT_OFF);
-            break;
-
-        case SQL_ATTR_CONNECTION_DEAD:
-            {
-                /*
-                  We have to check for both CR_SERVER_LOST and 
-                  CR_SERVER_GONE_ERROR to report about dead connections
-                */
-                if ( mysql_ping( &dbc->mysql ) && 
-                     (mysql_errno( &dbc->mysql ) == CR_SERVER_LOST ||
-                      mysql_errno( &dbc->mysql ) == CR_SERVER_GONE_ERROR) )
-                    *((SQLUINTEGER *) ValuePtr)= SQL_CD_TRUE;
-                else
-                    *((SQLUINTEGER *) ValuePtr)= SQL_CD_FALSE;
-            }
-            break;
-
-        case SQL_ATTR_CONNECTION_TIMEOUT:
-            /* We don't support this option, so it is always 0. */
-            *((SQLUINTEGER *) ValuePtr)= 0;
-            result= SQL_SUCCESS;
-            break;
-
-        case SQL_ATTR_CURRENT_CATALOG:
-
-            if (is_connected(dbc) && reget_current_catalog(dbc))
-            {
-                result= SQL_ERROR;
-            }
-            else
-            {
-                char *end= strmake((char*)ValuePtr,
-                                   dbc->database ? dbc->database : "null",
-                                   BufferLength);
-                *StringLengthPtr= (SQLSMALLINT) (end - (char*) ValuePtr);
-             }
-
-            break;
-
-        case SQL_ATTR_LOGIN_TIMEOUT:
-            *((SQLUINTEGER *) ValuePtr)= dbc->login_timeout;
-            break;
-
-        case SQL_ATTR_ODBC_CURSORS:
-            if ((dbc->flag & FLAG_FORWARD_CURSOR))
-                *((SQLUINTEGER *) ValuePtr)= SQL_CUR_USE_ODBC;
-            else
-                *((SQLUINTEGER *) ValuePtr)= SQL_CUR_USE_IF_NEEDED;
-            break;
-
-        case SQL_OPT_TRACE:
-        case SQL_OPT_TRACEFILE:
-        case SQL_QUIET_MODE:
-        case SQL_TRANSLATE_DLL:
-        case SQL_TRANSLATE_OPTION:
-            {
-                char buff[100];
-                sprintf(buff,
-                        "Suppose to get this attribute '%d' through driver manager, not by the driver",
-                        (int) Attribute);
-                result= set_conn_error(hdbc,MYERR_01S02,buff,0);
-                break;
-            }
-
-        case SQL_ATTR_PACKET_SIZE:
-            *((SQLUINTEGER *) ValuePtr)= dbc->mysql.net.max_packet;
-            break;
-
-        case SQL_ATTR_TXN_ISOLATION:
-            /*
-              If we don't know the isolation level already, we need
-              to ask the server.
-            */
-            if (!dbc->txn_isolation)
-            {
-              /*
-                Unless we're not connected yet, then we just assume it will
-                be REPEATABLE READ, which is the server default.
-              */
-              if (!is_connected(dbc))
-              {
-                *((SQLINTEGER *) ValuePtr)= SQL_TRANSACTION_REPEATABLE_READ;
-                break;
-              }
-
-              if (odbc_stmt(dbc, "SELECT @@tx_isolation"))
-              {
-                return set_handle_error(SQL_HANDLE_DBC,hdbc,
-                                        MYERR_S1000,
-                                        "Failed to get isolation level", 0);
-              }
-              else
-              {
-                  MYSQL_RES *res;
-                  MYSQL_ROW  row;
-
-                  if ((res= mysql_store_result(&dbc->mysql)) &&
-                      (row= mysql_fetch_row(res)))
-                  {
-                    if (strncmp(row[0], "READ-UNCOMMITTED", 16) == 0) {
-                      dbc->txn_isolation= SQL_TRANSACTION_READ_UNCOMMITTED;
-                    }
-                    else if (strncmp(row[0], "READ-COMMITTED", 14) == 0) {
-                      dbc->txn_isolation= SQL_TRANSACTION_READ_COMMITTED;
-                    }
-                    else if (strncmp(row[0], "REPEATABLE-READ", 15) == 0) {
-                      dbc->txn_isolation= SQL_TRANSACTION_REPEATABLE_READ;
-                    }
-                    else if (strncmp(row[0], "SERIALIZABLE", 12) == 0) {
-                      dbc->txn_isolation= SQL_TRANSACTION_SERIALIZABLE;
-                    }
-                  }
-                  mysql_free_result(res);
-              }
-            }
-
-            *((SQLINTEGER *) ValuePtr)= dbc->txn_isolation;
-            break;
-
-            /*
-              3.x driver doesn't support any statement attributes
-              at connection level, but to make sure all 2.x apps
-              works fine...lets support it..nothing to loose..
-            */
-        default:
-            result= get_constmt_attr(2,dbc,&dbc->stmt_options,
-                                     Attribute,ValuePtr,
-                                     StringLengthPtr);
+        if ((res= mysql_store_result(&dbc->mysql)) &&
+            (row= mysql_fetch_row(res)))
+        {
+          if (strncmp(row[0], "READ-UNCOMMITTED", 16) == 0) {
+            dbc->txn_isolation= SQL_TRANSACTION_READ_UNCOMMITTED;
+          }
+          else if (strncmp(row[0], "READ-COMMITTED", 14) == 0) {
+            dbc->txn_isolation= SQL_TRANSACTION_READ_COMMITTED;
+          }
+          else if (strncmp(row[0], "REPEATABLE-READ", 15) == 0) {
+            dbc->txn_isolation= SQL_TRANSACTION_REPEATABLE_READ;
+          }
+          else if (strncmp(row[0], "SERIALIZABLE", 12) == 0) {
+            dbc->txn_isolation= SQL_TRANSACTION_SERIALIZABLE;
+          }
+        }
+        mysql_free_result(res);
+      }
     }
-    return result;
+
+    *((SQLINTEGER *)num_attr)= dbc->txn_isolation;
+    break;
+
+  default:
+    return set_handle_error(SQL_HANDLE_DBC, hdbc, MYERR_S1092, NULL, 0);
+  }
+
+  return result;
 }
 
 
@@ -603,16 +514,16 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
   @purpose : sets the statement attributes
 */
 
-static SQLRETURN
-set_stmt_attr(SQLHSTMT   hstmt,
-              SQLINTEGER Attribute,
-              SQLPOINTER ValuePtr,
-              SQLINTEGER StringLengthPtr __attribute__((unused)))
+SQLRETURN SQL_API
+MySQLSetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute, SQLPOINTER ValuePtr,
+                 SQLINTEGER StringLengthPtr __attribute__((unused)))
 {
-    STMT FAR *stmt= (STMT FAR*) hstmt;
+    STMT *stmt= (STMT *)hstmt;
     SQLRETURN result= SQL_SUCCESS;
     STMT_OPTIONS *options= &stmt->stmt_options;
 
+    CLEAR_STMT_ERROR(stmt);
+
     switch (Attribute)
     {
         case SQL_ATTR_CURSOR_SCROLLABLE:
@@ -625,48 +536,153 @@ set_stmt_attr(SQLHSTMT   hstmt,
                 options->cursor_type= SQL_CURSOR_STATIC;
             break;
 
-        case SQL_ATTR_AUTO_IPD:
-            if (ValuePtr != (SQLPOINTER)SQL_FALSE)
-                return set_error(hstmt,MYERR_01S02,
-                                 "Option value changed to default auto ipd",0);
+        case SQL_ATTR_APP_PARAM_DESC:
+        case SQL_ATTR_APP_ROW_DESC:
+            {
+              DESC *desc= (DESC *) ValuePtr;
+              DESC **dest= NULL;
+              desc_desc_type desc_type;
+
+              /* reset to implicit if null */
+              if (desc == SQL_NULL_HANDLE)
+              {
+                if (Attribute == SQL_ATTR_APP_PARAM_DESC)
+                  stmt->apd= stmt->imp_apd;
+                else if (Attribute == SQL_ATTR_APP_ROW_DESC)
+                  stmt->ard= stmt->imp_ard;
+                break;
+              }
+
+              if (desc->alloc_type == SQL_DESC_ALLOC_AUTO &&
+                  desc->stmt != stmt)
+                return set_error(hstmt,MYERR_S1017,
+                                 "Invalid use of an automatically allocated "
+                                 "descriptor handle",0);
+
+              if (desc->alloc_type == SQL_DESC_ALLOC_USER &&
+                  stmt->dbc != desc->exp.dbc)
+                return set_error(hstmt,MYERR_S1024,
+                                 "Invalid attribute value",0);
+
+              if (Attribute == SQL_ATTR_APP_PARAM_DESC)
+              {
+                dest= &stmt->apd;
+                desc_type= DESC_PARAM;
+              }
+              else if (Attribute == SQL_ATTR_APP_ROW_DESC)
+              {
+                dest= &stmt->ard;
+                desc_type= DESC_ROW;
+              }
+
+              if (desc->desc_type != DESC_UNKNOWN &&
+                  desc->desc_type != desc_type)
+              {
+                return set_error(hstmt,MYERR_S1024,
+                                 "Descriptor type mismatch",0);
+              }
+
+              assert(desc);
+              assert(dest);
+
+              if (desc->alloc_type == SQL_DESC_ALLOC_AUTO &&
+                  (*dest)->alloc_type == SQL_DESC_ALLOC_USER)
+                /*
+                  If we're setting back the original implicit
+                  descriptor, we must disassociate this statement
+                  from the explicit descriptor.
+                */
+                desc_remove_stmt(*dest, stmt);
+              else if (desc->alloc_type == SQL_DESC_ALLOC_USER)
+              {
+                /* otherwise, associate this statement with the desc */
+                LIST *e= (LIST *) my_malloc(sizeof(LIST), MYF(0));
+                e->data= stmt;
+                desc->exp.stmts= list_add(desc->exp.stmts, e);
+              }
+
+              desc->desc_type= desc_type;
+              *dest= desc;
+            }
             break;
 
+        case SQL_ATTR_AUTO_IPD:
+        case SQL_ATTR_ENABLE_AUTO_IPD:
+            if (ValuePtr != (SQLPOINTER)SQL_FALSE)
+                return set_error(hstmt,MYERR_S1C00,
+                                 "Optional feature not implemented",0);
+            break;
+
+        case SQL_ATTR_IMP_PARAM_DESC:
+        case SQL_ATTR_IMP_ROW_DESC:
+            return set_error(hstmt,MYERR_S1024,
+                             "Invalid attribute/option identifier",0);
+
+        case SQL_ATTR_PARAM_BIND_OFFSET_PTR:
+            return stmt_SQLSetDescField(stmt, stmt->apd, 0,
+                                        SQL_DESC_BIND_OFFSET_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
+
+        case SQL_ATTR_PARAM_BIND_TYPE:
+            return stmt_SQLSetDescField(stmt, stmt->apd, 0,
+                                        SQL_DESC_BIND_TYPE,
+                                        ValuePtr, SQL_IS_INTEGER);
+
+        case SQL_ATTR_PARAM_OPERATION_PTR: /* need to support this ....*/
+            return stmt_SQLSetDescField(stmt, stmt->apd, 0,
+                                        SQL_DESC_ARRAY_STATUS_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
+
         case SQL_ATTR_PARAM_STATUS_PTR: /* need to support this ....*/
-            options->paramStatusPtr= (SQLUSMALLINT *)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ipd, 0,
+                                        SQL_DESC_ARRAY_STATUS_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
 
         case SQL_ATTR_PARAMS_PROCESSED_PTR: /* need to support this ....*/
-            options->paramProcessedPtr= (SQLUINTEGER *)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ipd, 0,
+                                        SQL_DESC_ROWS_PROCESSED_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
 
         case SQL_ATTR_PARAMSET_SIZE:
-            if (ValuePtr != (SQLPOINTER)1)
-                return set_error(hstmt,MYERR_01S02,
-                                 "Option value changed to default parameter size",
-                                 0);
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->apd, 0,
+                                        SQL_DESC_ARRAY_SIZE,
+                                        ValuePtr, SQL_IS_ULEN);
+
 
         case SQL_ATTR_ROW_ARRAY_SIZE:
         case SQL_ROWSET_SIZE:
-            options->rows_in_set= (SQLUINTEGER)(SQLULEN)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ard, 0,
+                                        SQL_DESC_ARRAY_SIZE,
+                                        ValuePtr, SQL_IS_ULEN);
+
+        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
+            return stmt_SQLSetDescField(stmt, stmt->ard, 0,
+                                        SQL_DESC_BIND_OFFSET_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
+
+        case SQL_ATTR_ROW_BIND_TYPE:
+            return stmt_SQLSetDescField(stmt, stmt->ard, 0,
+                                        SQL_DESC_BIND_TYPE,
+                                        ValuePtr, SQL_IS_INTEGER);
 
         case SQL_ATTR_ROW_NUMBER:
             return set_error(hstmt,MYERR_S1000,
                              "Trying to set read-only attribute",0);
-            break;
 
         case SQL_ATTR_ROW_OPERATION_PTR:
-            options->rowOperationPtr= (SQLUSMALLINT *)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ard, 0,
+                                        SQL_DESC_ARRAY_STATUS_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
 
         case SQL_ATTR_ROW_STATUS_PTR:
-            options->rowStatusPtr= (SQLUSMALLINT *)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ird, 0,
+                                        SQL_DESC_ARRAY_STATUS_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
 
         case SQL_ATTR_ROWS_FETCHED_PTR:
-            options->rowsFetchedPtr= (SQLULEN *)ValuePtr;
-            break;
+            return stmt_SQLSetDescField(stmt, stmt->ird, 0,
+                                        SQL_DESC_ROWS_PROCESSED_PTR,
+                                        ValuePtr, SQL_IS_POINTER);
 
         case SQL_ATTR_SIMULATE_CURSOR:
             options->simulateCursor= (SQLUINTEGER)(SQLULEN)ValuePtr;
@@ -675,7 +691,7 @@ set_stmt_attr(SQLHSTMT   hstmt,
             /*
               3.x driver doesn't support any statement attributes
               at connection level, but to make sure all 2.x apps
-              works fine...lets support it..nothing to loose..
+              works fine...lets support it..nothing to lose..
             */
         default:
             result= set_constmt_attr(3,hstmt,options,
@@ -690,16 +706,15 @@ set_stmt_attr(SQLHSTMT   hstmt,
   @purpose : returns the statement attribute values
 */
 
-static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
-                               SQLINTEGER Attribute,
-                               SQLPOINTER ValuePtr,
-                               SQLINTEGER BufferLength __attribute__((unused)),
-                               SQLINTEGER *StringLengthPtr)
+SQLRETURN SQL_API
+MySQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute, SQLPOINTER ValuePtr,
+                 SQLINTEGER BufferLength __attribute__((unused)),
+                 SQLINTEGER *StringLengthPtr)
 {
     SQLRETURN result= SQL_SUCCESS;
     STMT FAR *stmt= (STMT FAR*) hstmt;
     STMT_OPTIONS *options= &stmt->stmt_options;
-    SQLPOINTER vparam;
+    SQLINTEGER vparam= 0;
     SQLINTEGER len;
 
     if (!ValuePtr)
@@ -721,20 +736,41 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
             *(SQLUINTEGER *)ValuePtr= SQL_FALSE;
             break;
 
+        case SQL_ATTR_PARAM_BIND_OFFSET_PTR:
+            *(SQLPOINTER *)ValuePtr= stmt->apd->bind_offset_ptr;
+            break;
+
+        case SQL_ATTR_PARAM_BIND_TYPE:
+            *(SQLINTEGER *)ValuePtr= stmt->apd->bind_type;
+            break;
+
+        case SQL_ATTR_PARAM_OPERATION_PTR: /* need to support this ....*/
+            *(SQLPOINTER *)ValuePtr= stmt->apd->array_status_ptr;
+            break;
+
         case SQL_ATTR_PARAM_STATUS_PTR: /* need to support this ....*/
-            ValuePtr= (SQLUSMALLINT *)options->paramStatusPtr;
+            *(SQLPOINTER *)ValuePtr= stmt->ipd->array_status_ptr;
             break;
 
         case SQL_ATTR_PARAMS_PROCESSED_PTR: /* need to support this ....*/
-            ValuePtr= (SQLUSMALLINT *)options->paramProcessedPtr;
+            *(SQLPOINTER *)ValuePtr= stmt->ipd->rows_processed_ptr;
             break;
 
         case SQL_ATTR_PARAMSET_SIZE:
-            *(SQLUINTEGER *)ValuePtr= 1;
+            *(SQLUINTEGER *)ValuePtr= stmt->apd->array_size;
             break;
 
         case SQL_ATTR_ROW_ARRAY_SIZE:
-            *(SQLUINTEGER *)ValuePtr= options->rows_in_set;
+        case SQL_ROWSET_SIZE:
+            *(SQLUINTEGER *)ValuePtr= stmt->ard->array_size;
+            break;
+
+        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
+            *((SQLPOINTER *) ValuePtr)= stmt->ard->bind_offset_ptr;
+            break;
+
+        case SQL_ATTR_ROW_BIND_TYPE:
+            *((SQLINTEGER *) ValuePtr)= stmt->ard->bind_type;
             break;
 
         case SQL_ATTR_ROW_NUMBER:
@@ -742,51 +778,45 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
             break;
 
         case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-            ValuePtr= (SQLUSMALLINT *)options->rowOperationPtr;
+            *(SQLPOINTER *)ValuePtr= stmt->ard->array_status_ptr;
             break;
 
         case SQL_ATTR_ROW_STATUS_PTR:
-            ValuePtr= (SQLUSMALLINT *)options->rowStatusPtr;
+            *(SQLPOINTER *)ValuePtr= stmt->ird->array_status_ptr;
             break;
 
         case SQL_ATTR_ROWS_FETCHED_PTR:
-            ValuePtr= (SQLULEN *)options->rowsFetchedPtr;
+            *(SQLPOINTER *)ValuePtr= stmt->ird->rows_processed_ptr;
             break;
 
         case SQL_ATTR_SIMULATE_CURSOR:
             *(SQLUINTEGER *)ValuePtr= options->simulateCursor;
             break;
 
-            /*
-              To make iODBC and MS ODBC DM to work, return the following cases
-              as success, by just allocating...else
-              - iODBC is hanging at the time of stmt allocation
-              - MS ODB DM is crashing at the time of stmt allocation
-            */
         case SQL_ATTR_APP_ROW_DESC:
-            *(SQLPOINTER *)ValuePtr= &stmt->ard;
+            *(SQLPOINTER *)ValuePtr= stmt->ard;
             *StringLengthPtr= sizeof(SQLPOINTER);
             break;
 
         case SQL_ATTR_IMP_ROW_DESC:
-            *(SQLPOINTER *)ValuePtr= &stmt->ird;
+            *(SQLPOINTER *)ValuePtr= stmt->ird;
             *StringLengthPtr= sizeof(SQLPOINTER);
             break;
 
         case SQL_ATTR_APP_PARAM_DESC:
-            *(SQLPOINTER *)ValuePtr= &stmt->apd;
+            *(SQLPOINTER *)ValuePtr= stmt->apd;
             *StringLengthPtr= sizeof(SQLPOINTER);
             break;
 
         case SQL_ATTR_IMP_PARAM_DESC:
-            *(SQLPOINTER *)ValuePtr= &stmt->ipd;
+            *(SQLPOINTER *)ValuePtr= stmt->ipd;
             *StringLengthPtr= sizeof(SQLPOINTER);
             break;
 
             /*
               3.x driver doesn't support any statement attributes
               at connection level, but to make sure all 2.x apps
-              works fine...lets support it..nothing to loose..
+              works fine...lets support it..nothing to lose..
             */
         default:
             result= get_constmt_attr(3,hstmt,options,
@@ -798,56 +828,6 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
 }
 
 
-/*
-  @type    : ODBC 1.0 API
-  @purpose : sets the connection options
-*/
-
-SQLRETURN SQL_API SQLSetConnectOption( SQLHDBC      hdbc, 
-                                       SQLUSMALLINT fOption,
-                                       SQLULEN      vParam )
-{
-  return set_con_attr(hdbc, fOption, (SQLPOINTER)vParam, SQL_NTS);
-}
-
-
-/*
-  @type    : ODBC 1.0 API
-  @purpose : returns the connection options
-*/
-
-SQLRETURN SQL_API SQLGetConnectOption(SQLHDBC hdbc,SQLUSMALLINT fOption,
-                                      SQLPOINTER vParam)
-{
-  return get_con_attr(hdbc, fOption, vParam, SQL_NTS, (SQLINTEGER *)NULL);
-}
-
-
-/*
-  @type    : ODBC 1.0 API
-  @purpose : sets the statement options
-*/
-
-SQLRETURN SQL_API SQLSetStmtOption( SQLHSTMT        hstmt,
-                                    SQLUSMALLINT    fOption,
-                                    SQLULEN         vParam )
-{
-  return set_stmt_attr(hstmt,fOption,(SQLPOINTER)vParam,SQL_NTS);
-}
-
-
-/*
-  @type    : ODBC 1.0 API
-  @purpose : returns the statement options
-*/
-
-SQLRETURN SQL_API SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT fOption,
-                                   SQLPOINTER vParam)
-{
-  return get_stmt_attr(hstmt, fOption, vParam, SQL_NTS, (SQLINTEGER *)NULL);
-}
-
-
 /*
   @type    : ODBC 3.0 API
   @purpose : sets the environment attributes
@@ -913,56 +893,15 @@ SQLGetEnvAttr(SQLHENV    henv,
 }
 
 
-/*
-  @type    : ODBC 3.0 API
-  @purpose : sets the connection attributes
-*/
-
-SQLRETURN SQL_API SQLSetConnectAttr(SQLHDBC hdbc,
-                                    SQLINTEGER Attribute,
-                                    SQLPOINTER ValuePtr,
-                                    SQLINTEGER StringLength)
+SQLRETURN SQL_API
+SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT option, SQLPOINTER param)
 {
-  return set_con_attr(hdbc, Attribute, ValuePtr, StringLength);
+  return MySQLGetStmtAttr(hstmt, option, param, SQL_NTS, (SQLINTEGER *)NULL);
 }
 
-/*
-  @type    : ODBC 3.0 API
-  @purpose : returns the connection attribute values
-*/
 
-SQLRETURN SQL_API SQLGetConnectAttr(SQLHDBC hdbc,
-                                    SQLINTEGER Attribute,
-                                    SQLPOINTER ValuePtr,
-                                    SQLINTEGER BufferLength,
-                                    SQLINTEGER *StringLengthPtr)
+SQLRETURN SQL_API
+SQLSetStmtOption(SQLHSTMT hstmt, SQLUSMALLINT option, SQLULEN param)
 {
-  return get_con_attr(hdbc, Attribute, ValuePtr, BufferLength, StringLengthPtr);
-}
-
-/*
-  @type    : ODBC 3.0 API
-  @purpose : sets the statement attributes
-*/
-
-SQLRETURN SQL_API SQLSetStmtAttr(SQLHSTMT   hstmt,
-                                 SQLINTEGER Attribute,
-                                 SQLPOINTER ValuePtr,
-                                 SQLINTEGER StringLength)
-{
-  return set_stmt_attr(hstmt, Attribute, ValuePtr, StringLength );
-}
-
-/*
-  @type    : ODBC 3.0 API
-  @purpose : returns the statement attribute values
-*/
-SQLRETURN SQL_API SQLGetStmtAttr(SQLHSTMT   hstmt,
-                                 SQLINTEGER Attribute,
-                                 SQLPOINTER ValuePtr,
-                                 SQLINTEGER BufferLength,
-                                 SQLINTEGER *StringLengthPtr)
-{
-  return get_stmt_attr(hstmt ,Attribute, ValuePtr, BufferLength,
-                       StringLengthPtr);
+  return MySQLSetStmtAttr(hstmt, option, (SQLPOINTER)param, SQL_NTS);
 }
