@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -28,7 +28,7 @@ DECLARE_TEST(my_basics)
 {
   SQLLEN nRowCount;
 
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_basic");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_basic, t_basic_2");
 
   /* create the table 'myodbc3_demo_result' */
   ok_sql(hstmt,
@@ -267,8 +267,10 @@ DECLARE_TEST(charset_utf8)
 {
   HDBC hdbc1;
   HSTMT hstmt1;
-  SQLCHAR   conn[256], conn_out[256];
+  SQLCHAR conn[256], conn_out[256];
+  SQLLEN len;
   SQLSMALLINT conn_out_len;
+  SQLINTEGER str_size;
 
   /**
    Bug #19345: Table column length multiplies on size session character set
@@ -279,7 +281,7 @@ DECLARE_TEST(charset_utf8)
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-  sprintf((char *)conn, "DSN=%s;UID=%s;PASSWORD=%s;CHARSET=utf8",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=utf8",
           mydsn, myuid, mypwd);
   if (mysock != NULL)
   {
@@ -316,14 +318,48 @@ DECLARE_TEST(charset_utf8)
 
   ok_stmt(hstmt1, SQLFetch(hstmt1));
   is_num(my_fetch_int(hstmt1, 7), 10);
-  is_num(my_fetch_int(hstmt1, 8), 30);
-  is_num(my_fetch_int(hstmt1, 16), 30);
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 8, SQL_C_LONG, &str_size, 0, NULL));
+  /* utf8 mbmaxlen = 3 in libmysql before MySQL 6.0 */
+  if (str_size == 30)
+  {
+    is_num(my_fetch_int(hstmt1, 8), 30);
+    is_num(my_fetch_int(hstmt1, 16), 30);
+  }
+  else
+  {
+    is_num(my_fetch_int(hstmt1, 8), 40);
+    is_num(my_fetch_int(hstmt1, 16), 40);
+  }
 
   ok_stmt(hstmt1, SQLFetch(hstmt1));
   is_num(my_fetch_int(hstmt1, 7), 10);
   is_num(my_fetch_int(hstmt1, 8), 10);
   is_num(my_fetch_int(hstmt1, 16), 10);
 
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* Big5's 0xA4A4 becomes utf8's 0xE4B8AD */
+  ok_sql(hstmt1, "SELECT _big5 0xA4A4");
+
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_CHAR, conn, 2, &len));
+  is_num(conn[0], 0xE4);
+  is_num(len, 3);
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_CHAR, conn, 2, &len));
+  is_num(conn[0], 0xB8);
+  is_num(len, 2);
+
+  ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_CHAR, conn, 2, &len));
+  is_num(conn[0], 0xAD);
+  is_num(len, 1);
+
+  expect_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_CHAR, conn, 2, &len),
+              SQL_NO_DATA_FOUND);
+
+  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
+
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
   ok_con(hdbc1, SQLDisconnect(hdbc1));
   ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
@@ -353,7 +389,7 @@ DECLARE_TEST(charset_gbk)
   SQLCHAR str[]= "\xef\xbb\xbf\x27\xbf\x10";
   SQLSMALLINT conn_out_len;
 
-  sprintf((char *)conn, "DSN=%s;UID=%s;PASSWORD=%s;CHARSET=gbk",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=gbk",
           mydsn, myuid, mypwd);
   if (mysock != NULL)
   {
@@ -446,14 +482,14 @@ DECLARE_TEST(t_bug30774)
 {
   SQLHDBC hdbc1;
   SQLHSTMT hstmt1;
-  SQLCHAR username[80]= {0};
+  SQLCHAR username[MAX_ROW_DATA_LEN+1]= {0};
 
   strcat((char *)username, (char *)myuid);
   strcat((char *)username, "!!!");
 
   ok_env(henv, SQLAllocConnect(henv, &hdbc1));
   ok_con(hdbc1, SQLConnect(hdbc1, mydsn, SQL_NTS,
-                           username, strlen((char *)myuid),
+                           username, (SQLSMALLINT)strlen((char *)myuid),
                            mypwd, SQL_NTS));
   ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
 
@@ -514,13 +550,322 @@ DECLARE_TEST(t_bug30840)
 }
 
 
+/**
+  Bug #30983: SQL Statements limited to 64k
+*/
+DECLARE_TEST(t_bug30983)
+{
+  SQLCHAR buf[(80 * 1024) + 100]; /* ~80k */
+  SQLCHAR *bufp = buf;
+  SQLLEN buflen;
+  int i, j;
+
+  bufp+= sprintf((char *)bufp, "select '");
+
+  /* fill 1k of each value */
+  for (i= 0; i < 80; ++i)
+    for (j= 0; j < 512; ++j, bufp += 2)
+      sprintf((char *)bufp, "%02x", i);
+
+  sprintf((char *)bufp, "' as val");
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, buf, SQL_NTS));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, buf, 0, &buflen));
+  is_num(buflen, 80 * 1024);
+  return OK;
+}
+
+
+/*
+   Test the output string after calling SQLDriverConnect
+   Note: Windows 
+   TODO fix this test create a comparable output string
+*/
+DECLARE_TEST(t_driverconnect_outstring)
+{
+  HDBC hdbc1;
+  SQLCHAR conn[256], conn_out[256], exp_out[256];
+  SQLSMALLINT conn_out_len, exp_conn_out_len;
+
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=utf8",
+          mydsn, myuid, mypwd);
+  if (mysock != NULL)
+  {
+    strcat((char *)conn, ";SOCKET=");
+    strcat((char *)conn, (char *)mysock);
+  }
+
+  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
+
+  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
+                                 sizeof(conn_out), &conn_out_len,
+                                 SQL_DRIVER_NOPROMPT));
+
+  sprintf((char *)exp_out, "DSN=%s;UID=%s", mydsn, myuid);
+  if (mypwd && *mypwd)
+  {
+    strcat((char *)exp_out, ";PWD=");
+    strcat((char *)exp_out, (char *)mypwd);
+  }
+  strcat((char *)exp_out, ";DATABASE=");
+  ok_con(hdbc1, SQLGetConnectAttr(hdbc1, SQL_ATTR_CURRENT_CATALOG,
+                                  exp_out + strlen((char *)exp_out), 100,
+                                  NULL));
+
+  if (mysock != NULL)
+  {
+    strcat((char *)exp_out, ";SOCKET=");
+    strcat((char *)exp_out, (char *)mysock);
+  }
+  strcat((char *)exp_out, ";PORT=3306;CHARSET=utf8");
+
+  printMessage("Output connection string: %s", conn_out);
+  printMessage("Expected output   string: %s", exp_out);
+  /* save proper length for later tests */
+  exp_conn_out_len= conn_out_len;
+  is_num(conn_out_len, strlen((char *)conn_out));
+  /* TODO
+  is_str(conn_out, exp_out, strlen((char *)conn_out));
+  */
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+
+  /* test truncation */
+  conn_out_len= 999;
+  expect_dbc(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
+                                     10 * sizeof(SQLWCHAR), &conn_out_len,
+                                     SQL_DRIVER_NOPROMPT),
+             SQL_SUCCESS_WITH_INFO);
+  is_num(conn_out_len, 9);
+  is_num(check_sqlstate_ex(hdbc1, SQL_HANDLE_DBC, "01004"), OK);
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+
+  /* test truncation on boundary */
+  conn_out_len= 999;
+  expect_dbc(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
+                                     exp_conn_out_len * sizeof(SQLWCHAR),
+                                     &conn_out_len, SQL_DRIVER_NOPROMPT),
+             SQL_SUCCESS_WITH_INFO);
+  is_num(conn_out_len, exp_conn_out_len - 1);
+  is_num(check_sqlstate_ex(hdbc1, SQL_HANDLE_DBC, "01004"), OK);
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+  return OK;
+}
+
+
+DECLARE_TEST(setnames)
+{
+  expect_sql(hstmt, "SET NAMES utf8", SQL_ERROR);
+  expect_sql(hstmt, "SeT NamES utf8", SQL_ERROR);
+  expect_sql(hstmt, "   set names utf8", SQL_ERROR);
+  expect_sql(hstmt, "	set names utf8", SQL_ERROR);
+  return OK;
+}
+
+
+DECLARE_TEST(setnames_conn)
+{
+  HDBC hdbc1;
+  SQLCHAR conn[256], conn_out[256];
+  SQLSMALLINT conn_out_len;
+
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;INITSTMT={set names utf8}",
+          mydsn, myuid, mypwd);
+  if (mysock != NULL)
+  {
+    strcat((char *)conn, ";SOCKET=");
+    strcat((char *)conn, (char *)mysock);
+  }
+
+  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
+
+  expect_dbc(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
+                                     sizeof(conn_out), &conn_out_len,
+                                     SQL_DRIVER_NOPROMPT),
+             SQL_ERROR);
+
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+
+  return OK;
+}
+
+
+/**
+ Bug #15601: SQLCancel does not work to stop a query on the database server
+*/
+#ifdef WIN32
+DWORD WINAPI cancel_in_one_second(LPVOID arg)
+{
+  HSTMT hstmt= (HSTMT)arg;
+
+  Sleep(1000);
+
+  if (SQLCancel(hstmt) != SQL_SUCCESS)
+    printMessage("SQLCancel failed!");
+
+  return 0;
+}
+
+
+DECLARE_TEST(sqlcancel)
+{
+  HANDLE thread;
+  DWORD waitrc;
+
+  thread= CreateThread(NULL, 0, cancel_in_one_second, hstmt, 0, NULL);
+
+  /* SLEEP(n) returns 1 when it is killed. */
+  ok_sql(hstmt, "SELECT SLEEP(10)");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+
+  waitrc= WaitForSingleObject(thread, 10000);
+  is(!(waitrc == WAIT_TIMEOUT));
+
+  return OK;
+}
+#else
+void *cancel_in_one_second(void *arg)
+{
+  HSTMT *hstmt= arg;
+
+  sleep(1);
+
+  if (SQLCancel(hstmt) != SQL_SUCCESS)
+    printMessage("SQLCancel failed!");
+
+  return NULL;
+}
+
+#include <pthread.h>
+
+DECLARE_TEST(sqlcancel)
+{
+  pthread_t thread;
+
+  pthread_create(&thread, NULL, cancel_in_one_second, hstmt);
+
+  /* SLEEP(n) returns 1 when it is killed. */
+  ok_sql(hstmt, "SELECT SLEEP(10)");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+
+  pthread_join(thread, NULL);
+
+  return OK;
+}
+#endif
+
+
+/**
+Bug #32014: MyODBC / ADO Unable to open record set using dynamic cursor
+*/
+DECLARE_TEST(t_bug32014)
+{
+  SQLHENV     henv1;
+  SQLHDBC     hdbc1;
+  SQLHSTMT    hstmt1;
+  SQLUINTEGER info;
+  long        i=0;
+  SQLSMALLINT value_len;
+
+  long flags[]= { 0,
+                  (131072L << 4)   /*FLAG_FORWARD_CURSOR*/,
+                  32               /*FLAG_DYNAMIC_CURSOR*/,
+                  (131072L << 4) | 32,
+                  0 };
+
+  long expectedInfo[]= { SQL_SO_FORWARD_ONLY|SQL_SO_STATIC,
+                         SQL_SO_FORWARD_ONLY,
+                         SQL_SO_FORWARD_ONLY|SQL_SO_STATIC|SQL_SO_DYNAMIC,
+                         SQL_SO_FORWARD_ONLY };
+
+  long expectedCurType[][4]= {
+      {SQL_CURSOR_FORWARD_ONLY, SQL_CURSOR_STATIC,        SQL_CURSOR_STATIC,          SQL_CURSOR_STATIC},
+      {SQL_CURSOR_FORWARD_ONLY, SQL_CURSOR_FORWARD_ONLY,  SQL_CURSOR_FORWARD_ONLY,    SQL_CURSOR_FORWARD_ONLY},
+      {SQL_CURSOR_FORWARD_ONLY, SQL_CURSOR_STATIC,        SQL_CURSOR_DYNAMIC,         SQL_CURSOR_STATIC},
+      {SQL_CURSOR_FORWARD_ONLY, SQL_CURSOR_FORWARD_ONLY,  SQL_CURSOR_FORWARD_ONLY,    SQL_CURSOR_FORWARD_ONLY}};
+
+  do
+  {
+    SET_DSN_OPTION(flags[i]);
+    alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+    printMessage("checking %d (%d)", i, flags[i]);
+
+    /*Checking that correct info is returned*/
+
+    ok_stmt(hstmt1, SQLGetInfo(hdbc1, SQL_SCROLL_OPTIONS,
+            (SQLPOINTER) &info, sizeof(long), &value_len));
+    is_num(info, expectedInfo[i]);
+
+    /*Checking that correct cursor type is set*/
+
+    ok_stmt(hstmt1, SQLSetStmtOption(hstmt1, SQL_CURSOR_TYPE
+            , SQL_CURSOR_FORWARD_ONLY ));
+    ok_stmt(hstmt1, SQLGetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            (SQLPOINTER) &info));
+    is_num(info, expectedCurType[i][SQL_CURSOR_FORWARD_ONLY]);
+
+    ok_stmt(hstmt1, SQLSetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            SQL_CURSOR_KEYSET_DRIVEN ));
+    ok_stmt(hstmt1, SQLGetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            (SQLPOINTER) &info));
+    is_num(info, expectedCurType[i][SQL_CURSOR_KEYSET_DRIVEN]);
+
+    ok_stmt(hstmt1, SQLSetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            SQL_CURSOR_DYNAMIC ));
+    ok_stmt(hstmt1, SQLGetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            (SQLPOINTER) &info));
+    is_num(info, expectedCurType[i][SQL_CURSOR_DYNAMIC]);
+
+    ok_stmt(hstmt1, SQLSetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            SQL_CURSOR_STATIC ));
+    ok_stmt(hstmt1, SQLGetStmtOption(hstmt1, SQL_CURSOR_TYPE,
+            (SQLPOINTER) &info));
+    is_num(info, expectedCurType[i][SQL_CURSOR_STATIC]);
+
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  } while (flags[++i]);
+
+  SET_DSN_OPTION(0);
+
+  return OK;
+}
+
+
+/*
+  Bug #10128 Error in evaluating simple mathematical expression
+  ADO calls SQLNativeSql with a NULL pointer for the result length,
+  but passes a non-NULL result buffer.
+*/
+DECLARE_TEST(t_bug10128)
+{
+  SQLCHAR *query= (SQLCHAR *) "select 1,2,3,4";
+  SQLCHAR nativesql[1000];
+  SQLINTEGER nativelen;
+  SQLINTEGER querylen= (SQLINTEGER) strlen((char *)query);
+
+  ok_con(hdbc, SQLNativeSql(hdbc, query, SQL_NTS, NULL, 0, &nativelen));
+  is_num(nativelen, querylen);
+
+  ok_con(hdbc, SQLNativeSql(hdbc, query, SQL_NTS, nativesql, 1000, NULL));
+  is_str(nativesql, query, querylen + 1);
+
+  return OK;
+}
+
+
 /**
  Bug #32727: Unable to abort distributed transactions enlisted in MSDTC
 */
 DECLARE_TEST(t_bug32727)
 {
-  is(SQLSetConnectAttr(hdbc, SQL_ATTR_ENLIST_IN_DTC,
-                       (SQLPOINTER)1, SQL_IS_UINTEGER) == SQL_ERROR);
+  is_num(SQLSetConnectAttr(hdbc, SQL_ATTR_ENLIST_IN_DTC,
+                       (SQLPOINTER)1, SQL_IS_UINTEGER), SQL_ERROR);
   return OK;
 }
 
@@ -531,6 +876,7 @@ DECLARE_TEST(t_bug32727)
 DECLARE_TEST(t_bug28820)
 {
   SQLULEN length;
+  SQLCHAR dummy[20];
   SQLSMALLINT i;
 
   ok_sql(hstmt, "drop table if exists t_bug28820");
@@ -544,7 +890,7 @@ DECLARE_TEST(t_bug28820)
   for (i= 0; i < 3; ++i)
   {
     length= 0;
-    ok_stmt(hstmt, SQLDescribeCol(hstmt, i+1, NULL, 0, NULL,
+    ok_stmt(hstmt, SQLDescribeCol(hstmt, i+1, dummy, sizeof(dummy), NULL,
                                   NULL, &length, NULL, NULL));
     is_num(length, 90);
   }
@@ -564,10 +910,13 @@ DECLARE_TEST(t_bug31959)
   SQLINTEGER i;
   SQLINTEGER levelid[] = {SQL_TXN_SERIALIZABLE, SQL_TXN_REPEATABLE_READ,
                           SQL_TXN_READ_COMMITTED, SQL_TXN_READ_UNCOMMITTED};
-  SQLCHAR *levelname[] = {"SERIALIZABLE", "REPEATABLE-READ",
-                          "READ-COMMITTED", "READ-UNCOMMITTED"};
+  SQLCHAR *levelname[] = {(SQLCHAR *)"SERIALIZABLE",
+                          (SQLCHAR *)"REPEATABLE-READ",
+                          (SQLCHAR *)"READ-COMMITTED",
+                          (SQLCHAR *)"READ-UNCOMMITTED"};
 
-  ok_stmt(hstmt, SQLPrepare(hstmt, "select @@tx_isolation", SQL_NTS));
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"select @@tx_isolation", SQL_NTS));
 
   /* check all 4 valid isolation levels */
   for(i = 3; i >= 0; --i)
@@ -577,13 +926,13 @@ DECLARE_TEST(t_bug31959)
     ok_stmt(hstmt, SQLExecute(hstmt));
     ok_stmt(hstmt, SQLFetch(hstmt));
     ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, level, 50, NULL));
-    is_str(level, levelname[i], strlen(levelname[i]));
+    is_str(level, levelname[i], strlen((char *)levelname[i]));
     printMessage("Level = %s\n", level);
     ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   }
 
   /* check invalid value (and corresponding SQL state) */
-  is(SQLSetConnectAttr(hdbc, SQL_ATTR_TXN_ISOLATION, (SQLPOINTER)999, 0) ==
+  is_num(SQLSetConnectAttr(hdbc, SQL_ATTR_TXN_ISOLATION, (SQLPOINTER)999, 0),
      SQL_ERROR);
   {
   SQLCHAR     sql_state[6];
@@ -602,6 +951,45 @@ DECLARE_TEST(t_bug31959)
 }
 
 
+/*
+  Bug #41256 - NULL parameters don't work correctly with ADO.
+  The null indicator pointer can be set separately through the
+  descriptor field. This wasn't being checked separately.
+*/
+DECLARE_TEST(t_bug41256)
+{
+  SQLHANDLE apd;
+  SQLINTEGER val= 40;
+  SQLLEN vallen= 19283;
+  SQLLEN ind= SQL_NULL_DATA;
+  SQLLEN reslen= 40;
+  ok_stmt(hstmt, SQLGetStmtAttr(hstmt, SQL_ATTR_APP_PARAM_DESC,
+                                &apd, SQL_IS_POINTER, NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_INTEGER,
+                                  SQL_C_LONG, 0, 0, &val, 0, &vallen));
+  ok_desc(apd, SQLSetDescField(apd, 1, SQL_DESC_INDICATOR_PTR,
+                               &ind, SQL_IS_POINTER));
+  ok_sql(hstmt, "select ?");
+  val= 80;
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_LONG, &val, 0, &reslen));
+  is_num(SQL_NULL_DATA, reslen);
+  is_num(80, val);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  return OK;
+}
+
+
+DECLARE_TEST(t_bug44971)
+{
+/*  ok_sql(hstmt, "drop database if exists bug44971");
+  ok_sql(hstmt, "create database bug44971");
+  ok_con(hdbc, SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, "bug44971xxx", 8));
+  ok_sql(hstmt, "drop database if exists bug44971");*/
+  return OK;
+}
+
+
 DECLARE_TEST(t_bug48603)
 {
   SQLINTEGER timeout, interactive, diff= 1000;
@@ -609,40 +997,35 @@ DECLARE_TEST(t_bug48603)
   HDBC hdbc1;
   HSTMT hstmt1;
   SQLCHAR conn[256], conn_out[256], query[53];
+  int result= OK;
 
   ok_sql(hstmt, "select @@wait_timeout, @@interactive_timeout");
   ok_stmt(hstmt,SQLFetch(hstmt));
-
-  timeout=      my_fetch_int(hstmt, 1);
-  interactive=  my_fetch_int(hstmt, 2);
-
+  timeout= my_fetch_int(hstmt, 1);
+  interactive= my_fetch_int(hstmt, 2);
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   if (timeout == interactive)
   {
-    printMessage("Changing interactive timeout globally as it is equal to wait_timeout");
     /* Changing globally interactive timeout to be able to test
        if INTERACTIVE option works */
+    SQLRETURN rc;
     sprintf((char *)query, "set GLOBAL interactive_timeout=%d", timeout + diff);
+    rc= SQLExecDirect(hstmt, query, SQL_NTS);
 
-    if (!SQL_SUCCEEDED(SQLExecDirect(hstmt, query, SQL_NTS)))
+    if (!SQL_SUCCEEDED(rc))
     {
       printMessage("Don't have rights to change interactive timeout globally - so can't really test if option INTERACTIVE works");
-      // Let the testcase does not fail
-      diff= 0;
       //return FAIL;
     }
 
     ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   }
   else
-  {
-    printMessage("Interactive: %d, wait: %d", interactive, timeout);
     diff= interactive - timeout;
-  }
 
   /* INITSTMT={set @@wait_timeout=%d} */
-  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=utf8;INITSTMT=set @@interactive_timeout=%d;INTERACTIVE=1",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=utf8;INITSTMT={set @@interactive_timeout=%d};INTERACTIVE=1",
     mydsn, myuid, mypwd, timeout+diff);
 
   if (mysock != NULL)
@@ -662,86 +1045,56 @@ DECLARE_TEST(t_bug48603)
   ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
     sizeof(conn_out), &conn_out_len,
     SQL_DRIVER_NOPROMPT));
-
   ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
   ok_sql(hstmt1, "select @@wait_timeout");
   ok_stmt(hstmt1,SQLFetch(hstmt1));
 
+  if ((timeout+diff) != my_fetch_int(hstmt1, 1))
+    result= FAIL;
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+
+  if (timeout == interactive)
   {
-    SQLINTEGER cur_timeout= my_fetch_int(hstmt1, 1);
-
-    ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
-    ok_con(hdbc1, SQLDisconnect(hdbc1));
-    ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
-
-    if (timeout == interactive)
-    {
-      /* setting global interactive timeout back if we changed it */
-      sprintf((char *)query, "set GLOBAL interactive_timeout=%d", timeout);
-      ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
-    }
-
-    is_num(timeout + diff, cur_timeout);
+    /* setting global interactive timeout back if we changed it */
+    sprintf((char *)query, "set GLOBAL interactive_timeout=%d", timeout);
+    ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
   }
 
-  return OK;
+  return result;
 }
 
-#ifdef _WIN32
-//#define NOT_AUTOMATED_TESTING
-#ifdef NOT_AUTOMATED_TESTING
 
-/* This is only to facilitate manual testing of prompt function */
-DECLARE_TEST(test_prompt)
+/*
+  Bug#45378 - spaces in connection string aren't removed
+*/
+DECLARE_TEST(t_bug45378)
 {
-  HDBC        hdbc1;
-  SQLCHAR     conn[256], conn_out[256];
+  HDBC hdbc1;
+  SQLCHAR conn[256], conn_out[256];
   SQLSMALLINT conn_out_len;
-  HWND        consoleHwnd;
-  LPCSTR      winTitle= "MyODBCPromptTest";
 
-  /* For manual testing we don't need super unique title and restore the title later */
-  SetConsoleTitle(winTitle);
-  /**
-   Bug #19345: Table column length multiplies on size session character set
-  */
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug19345");
-  ok_sql(hstmt, "CREATE TABLE t_bug19345 (a VARCHAR(10), b VARBINARY(10))");
-  ok_sql(hstmt, "INSERT INTO t_bug19345 VALUES ('abc','def')");
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  sprintf((char *)conn, "DSN=%s;UID=%s;PASSWORD=%s;CHARSET=utf8",
+  sprintf((char *)conn, "DSN=%s; UID = {%s} ;PWD= %s ",
           mydsn, myuid, mypwd);
   if (mysock != NULL)
   {
     strcat((char *)conn, ";SOCKET=");
     strcat((char *)conn, (char *)mysock);
   }
-  if (myport)
-  {
-    char pbuff[20];
-    sprintf(pbuff, ";PORT=%d", myport);
-    strcat((char *)conn, pbuff);
-  }
 
   ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
 
-  while ( !(consoleHwnd= FindWindow(NULL, winTitle)) );
-
-  ok_con(hdbc1, SQLDriverConnect(hdbc1, consoleHwnd, conn, sizeof(conn), conn_out,
+  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
                                  sizeof(conn_out), &conn_out_len,
-                                 SQL_DRIVER_PROMPT));
-
-  printMessage("In: %s, Out: %s", conn, conn_out);
-
+                                 SQL_DRIVER_NOPROMPT));
   ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeConnect(hdbc1));
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
 
   return OK;
 }
-#endif
-#endif
+
 
 BEGIN_TESTS
   ADD_TEST(my_basics)
@@ -757,13 +1110,20 @@ BEGIN_TESTS
   ADD_TEST(t_bug7445)
   ADD_TEST(t_bug30774)
   ADD_TEST(t_bug30840)
+  ADD_TEST(t_bug30983)
+  ADD_TEST(t_driverconnect_outstring)
+  ADD_TEST(setnames)
+  ADD_TEST(setnames_conn)
+  ADD_TEST(sqlcancel)
+  ADD_TEST(t_bug32014)
+  ADD_TEST(t_bug10128)
   ADD_TEST(t_bug32727)
   ADD_TEST(t_bug28820)
   ADD_TEST(t_bug31959)
+  ADD_TEST(t_bug41256)
+  ADD_TEST(t_bug44971)
   ADD_TEST(t_bug48603)
-#if defined(_WIN32) && defined(NOT_AUTOMATED_TESTING)
-  ADD_TEST(test_prompt)
-#endif
+  ADD_TEST(t_bug45378)
 END_TESTS
 
 
