@@ -1,29 +1,25 @@
 /*
-  Copyright 2001, 2011, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the
-  GPL, like most MySQL Connectors. There are special exceptions
-  to the terms and conditions of the GPL as it is applied to
-  this software, see the FLOSS License Exception available on
-  mysql.com.
+  Copyright (c) 2001, 2010, Oracle and/or its affiliates. All rights reserved.
 
+  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+  MySQL Connectors. There are special exceptions to the terms and
+  conditions of the GPLv2 as it is applied to this software, see the
+  FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+  
   This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License as
-  published by the Free Software Foundation.
-
-  There are special exceptions to the terms and conditions of the GPL
-  as it is applied to this software. View the full text of the exception
-  in file LICENSE.exceptions in the top-level directory of this software
-  distribution.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  it under the terms of the GNU General Public License as published
+  by the Free Software Foundation; version 2 of the License.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+  for more details.
+  
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "odbctap.h"
@@ -34,7 +30,7 @@ DECLARE_TEST(my_resultset)
     SQLRETURN   rc;
     SQLUINTEGER nRowCount=0;
     SQLULEN     pcColDef;
-    SQLCHAR     szColName[MAX_NAME_LEN];
+    SQLCHAR     szColName[MAX_NAME_LEN+1];
     SQLCHAR     szData[MAX_ROW_DATA_LEN+1];
     SQLSMALLINT nIndex,ncol,pfSqlType, pcbScale, pfNullable;
 
@@ -218,7 +214,8 @@ DECLARE_TEST(t_convert_type)
       mystmt(hstmt,rc);
 
       printMessage("CAST(MAX(col1) AS CHAR): %d", SqlType);
-      myassert(SqlType == SQL_VARCHAR || SqlType == SQL_LONGVARCHAR);
+      myassert(SqlType == SQL_VARCHAR || SqlType == SQL_LONGVARCHAR ||
+               SqlType == SQL_WVARCHAR || SqlType == SQL_WLONGVARCHAR);
 
       SQLFreeStmt(hstmt,SQL_CLOSE);
     }
@@ -297,7 +294,7 @@ DECLARE_TEST(t_desc_col)
          "c20 blob,"
          "c21 mediumblob,"
          "c22 longblob,"
-         "c23 tinyblob)");
+         "c23 tinyblob) CHARSET latin1");
 
   ok_sql(hstmt, "SELECT * FROM t_desc_col");
 
@@ -324,11 +321,11 @@ DECLARE_TEST(t_desc_col)
   is(desc_col_check(hstmt, 15, "c15", SQL_LONGVARCHAR, 255, 255, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 16, "c16", SQL_LONGVARCHAR, 65535, 65535, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 17, "c17", SQL_LONGVARCHAR, 16777215, 16777215, 0,  SQL_NULLABLE) == OK);
-  is(desc_col_check(hstmt, 18, "c18", SQL_LONGVARCHAR, 4294967295 , 16777215 , 0,  SQL_NULLABLE) == OK);
+  is(desc_col_check(hstmt, 18, "c18", SQL_LONGVARCHAR, 4294967295UL, 16777215 , 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 19, "c19", SQL_LONGVARBINARY, 255, 255, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 20, "c20", SQL_LONGVARBINARY, 65535, 65535, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 21, "c21", SQL_LONGVARBINARY, 16777215, 16777215, 0,  SQL_NULLABLE) == OK);
-  is(desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295 , 16777215 , 0,  SQL_NULLABLE) == OK);
+  is(desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295UL, 16777215 , 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 23, "c23", SQL_LONGVARBINARY, 255, 5, 0,  SQL_NULLABLE) == OK);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -398,6 +395,7 @@ DECLARE_TEST(t_max_rows)
 {
   SQLRETURN rc;
   SQLUINTEGER i;
+  SQLSMALLINT cc;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_max_rows");
 
@@ -466,6 +464,27 @@ DECLARE_TEST(t_max_rows)
 
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
+    /* Patch for Bug#46411 uses SQL_ATTR_MAX_ROWS attribute to minimize number of
+       rows to pre-fetch(sets to 1). Following fragment ensures that attribute's
+       value is preserved and works. */
+    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_MAX_ROWS,(SQLPOINTER)3,0);
+    mystmt(hstmt,rc);
+
+    rc = tmysql_prepare(hstmt,"select * from t_max_rows where id > ?");
+    mystmt(hstmt,rc);
+    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_ULONG,SQL_INTEGER,0,0,&i,0,NULL);
+    mystmt(hstmt,rc);
+    i= 6;
+
+    ok_stmt(hstmt, SQLNumResultCols(hstmt, &cc));
+
+    rc = SQLExecute(hstmt);
+    mystmt(hstmt,rc);
+
+    myassert( 3 == myrowcount(hstmt));
+
+    SQLFreeStmt(hstmt,SQL_CLOSE);
+
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_MAX_ROWS,(SQLPOINTER)0,0);
     mystmt(hstmt,rc);
 
@@ -580,9 +599,7 @@ DECLARE_TEST(t_multistep)
     myassert(pcbValue == 99);
     myassert(szData[0] == 'A');
 
-
-    rc = SQLFetch(hstmt);
-    myassert(rc == SQL_NO_DATA_FOUND);
+    expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
     SQLFreeStmt(hstmt,SQL_UNBIND);
     SQLFreeStmt(hstmt,SQL_CLOSE);
@@ -625,22 +642,19 @@ DECLARE_TEST(t_zerolength)
     mystmt(hstmt,rc);
 
     pcbValue= pcbValue1= 99;
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,szData,0,&pcbValue);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szData, 0, &pcbValue));
     printMessage("length: %d", pcbValue);
     myassert(pcbValue == 0);
 
     bData[0]=bData[1]='z';
-    rc = SQLGetData(hstmt,2,SQL_C_BINARY,bData,0,&pcbValue1);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_BINARY, bData, 0, &pcbValue1));
     printMessage("length: %d", pcbValue1);
     myassert(pcbValue1 == 0);
     myassert(bData[0] == 'z');
     myassert(bData[1] == 'z');
 
     bData1[0]=bData1[1]='z';
-    rc = SQLGetData(hstmt,3,SQL_C_BINARY,bData1,0,&pcbValue2);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_BINARY, bData1, 0, &pcbValue2));
     printMessage("length: %d", pcbValue2);
     myassert(pcbValue2 == 0);
     myassert(bData1[0] == 'z');
@@ -658,7 +672,6 @@ DECLARE_TEST(t_zerolength)
     mystmt(hstmt,rc);
     printMessage("data: %s, length: %d", bData, pcbValue1);
     myassert(pcbValue1 == 0);
-    myassert(bData[0]== '\0');
 
     bData1[0]=bData1[1]='z';
     rc = SQLGetData(hstmt,3,SQL_C_CHAR,bData1,1,&pcbValue2);
@@ -792,7 +805,7 @@ DECLARE_TEST(t_cache_bug)
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=%s;OPTION=1048579",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=1048579",
           mydsn, myuid, mypwd, mydb);
   if (mysock != NULL)
   {
@@ -859,7 +872,7 @@ DECLARE_TEST(t_non_cache_bug)
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=%s;OPTION=3",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=3",
           mydsn, myuid, mypwd, mydb);
   if (mysock != NULL)
   {
@@ -1137,7 +1150,8 @@ DECLARE_TEST(t_desccolext)
         n2 numeric(10,3),\
         dt date,\
         dtime datetime,\
-        ts timestamp,\
+        ts1 timestamp(8),\
+        ts2 timestamp(14),\
         ti  time,\
         yr1 year,\
         yr2 year(2),\
@@ -1269,8 +1283,7 @@ DECLARE_TEST(t_desccol1)
 
 DECLARE_TEST(t_colattributes)
 {
-  SQLLEN count;
-  SQLINTEGER isauto, issensitive;
+  SQLLEN count, isauto;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_colattr");
 
@@ -1285,7 +1298,7 @@ DECLARE_TEST(t_colattributes)
          "m1 MEDIUMINT,"
          "m2 MEDIUMINT(10),"
          "m3 MEDIUMINT UNSIGNED,"
-/* 10 */ "i1 INT,"
+         "i1 INT,"
          "i2 INT(10) NOT NULL,"
          "i3 INT UNSIGNED,"
          "i4 INT ZEROFILL,"
@@ -1295,7 +1308,7 @@ DECLARE_TEST(t_colattributes)
          "f1 FLOAT,"
          "f2 FLOAT(10),"
          "f3 FLOAT(24) ZEROFILL,"
-/* 20 */ "f4 FLOAT(10,4),"
+         "f4 FLOAT(10,4),"
          "d1 DOUBLE,"
          "d2 DOUBLE(30,3),"
          "d3 DOUBLE PRECISION,"
@@ -1305,17 +1318,17 @@ DECLARE_TEST(t_colattributes)
          "dc1 DECIMAL,"
          "dc2 DECIMAL(10),"
          "dc3 DECIMAL(10,3),"
-/* 30 */ "n1 NUMERIC,"
+         "n1 NUMERIC,"
          "n2 NUMERIC(10,3),"
          "dt DATE,"
          "dtime DATETIME,"
-         "ts1 TIMESTAMP/*(8)*/,"
-         "ts2 TIMESTAMP/*(14)*/,"
+         "ts1 TIMESTAMP(8),"
+         "ts2 TIMESTAMP(14),"
          "ti  TIME,"
          "yr1 YEAR,"
          "yr2 YEAR(2),"
          "yr3 YEAR(4),"
-/* 40 */ "c1 CHAR(10),"
+         "c1 CHAR(10),"
          "c2 CHAR(10) BINARY,"
          "c3 NATIONAL CHAR(10),"
          "v1 VARCHAR(10),"
@@ -1325,7 +1338,7 @@ DECLARE_TEST(t_colattributes)
          "bl2 BLOB,"
          "bl3 MEDIUMBLOB,"
          "bl4 LONGBLOB,"
-/* 50 */ "txt1 TINYTEXT,"
+         "txt1 TINYTEXT,"
          "txt2 TEXT,"
          "txt3 MEDIUMTEXT,"
          "txt4 LONGTEXT,"
@@ -1343,17 +1356,6 @@ DECLARE_TEST(t_colattributes)
                                  NULL, &isauto));
   is_num(isauto, SQL_TRUE);
 
-
-  /* c1 CHAR(10) */
-  ok_stmt(hstmt, SQLColAttribute(hstmt, 40, SQL_DESC_CASE_SENSITIVE,NULL, 0, NULL, &issensitive));
-  is_num(issensitive, SQL_FALSE);
-  /* c2 CHAR(10) BINARY */
-  ok_stmt(hstmt, SQLColAttribute(hstmt, 41, SQL_DESC_CASE_SENSITIVE,NULL, 0, NULL, &issensitive));
-  is_num(issensitive, SQL_TRUE);
-  /* bl1 TINYBLOB */
-  ok_stmt(hstmt, SQLColAttribute(hstmt, 46, SQL_DESC_CASE_SENSITIVE,NULL, 0, NULL, &issensitive));
-  is_num(issensitive, SQL_TRUE);
-
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_colattr");
 
   return OK;
@@ -1594,6 +1596,7 @@ DECLARE_TEST(t_true_length)
   SQLCHAR data1[25],data2[25];
   SQLLEN len1,len2;
   SQLULEN desc_len;
+  SQLSMALLINT name_len;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_true_length");
   ok_sql(hstmt, "CREATE TABLE t_true_length (a CHAR(20), b VARCHAR(15))");
@@ -1603,11 +1606,11 @@ DECLARE_TEST(t_true_length)
 
   ok_sql(hstmt, "SELECT * FROM t_true_length");
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 0, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, data1, sizeof(data1), &name_len, NULL,
                                 &desc_len, NULL, NULL));
   is_num(desc_len, 20);
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, NULL, 0, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, data1, sizeof(data1), &name_len, NULL,
                                 &desc_len, NULL, NULL));
   is_num(desc_len, 15);
 
@@ -1754,13 +1757,13 @@ DECLARE_TEST(t_binary_collation)
                                 &decimal_digits, &nullable));
   if (mysql_min_version(hdbc, "5.2", 3) ||
       /* 5.0.46 or later in 5.0 series */
-      (!strncmp("5.0", server_version, 3) &&
+      (!strncmp("5.0", (char *)server_version, 3) &&
         mysql_min_version(hdbc, "5.0.46", 6)) ||
       /* 5.1.22 or later in 5.1 series */
-      (!strncmp("5.1", server_version, 3) &&
+      (!strncmp("5.1", (char *)server_version, 3) &&
         mysql_min_version(hdbc, "5.1.22", 6)))
   {
-    is_num(data_type, SQL_VARCHAR);
+    is_num(data_type, SQL_WVARCHAR);
   }
   else
   {
@@ -1777,7 +1780,7 @@ DECLARE_TEST(t_binary_collation)
 DECLARE_TEST(t_bug29239)
 {
   SQLHANDLE hstmt2;
-  SQLINTEGER xval;
+  SQLINTEGER xval = 88;
 
   ok_sql(hstmt, "drop table if exists bug29239");
   ok_sql(hstmt, "create table bug29239 ( x int )");
@@ -1798,6 +1801,8 @@ DECLARE_TEST(t_bug29239)
   is_num(xval, 44);
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
   ok_sql(hstmt, "drop table if exists bug29239");
   return OK;
 }
@@ -1862,6 +1867,120 @@ DECLARE_TEST(t_bug30958)
 }
 
 
+/**
+ A variant of the above test, with charset != ansi charset
+*/
+DECLARE_TEST(t_bug30958_ansi)
+{
+  SQLCHAR outbuf[20]= "bug";
+  SQLLEN outlen;
+  SQLINTEGER outmax= 0;
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+  ok_sql(hstmt, "CREATE TABLE bug30958 (tt_textfield TEXT CHARACTER SET latin2 NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO bug30958 (tt_textfield) VALUES ('')");
+
+  ok_sql(hstmt, "select tt_textfield from bug30958");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /*
+   check first that we get truncation, with zero bytes
+   available in out buffer, outbuffer should be untouched
+  */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* expect the same result, and not SQL_NO_DATA */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* now provide a space to read the data */
+  outmax= 1;
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax, &outlen));
+  is_num(outbuf[0], 0);
+  is_num(outlen, 0);
+
+  /* only now is it unavailable (test with empty and non-empty out buffer) */
+  outmax= 0;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+  outmax= 1;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+
+  return OK;
+}
+
+
+/**
+ A variant of the above tests using SQLWVARCHAR
+*/
+DECLARE_TEST(t_bug30958_wchar)
+{
+  SQLCHAR outbuf[20]= "bug";
+  SQLLEN outlen;
+  SQLINTEGER outmax= 0;
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+  ok_sql(hstmt, "CREATE TABLE bug30958 (tt_textfield TEXT NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO bug30958 (tt_textfield) VALUES ('')");
+
+  ok_sql(hstmt, "select tt_textfield from bug30958");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /*
+   check first that we get truncation, with zero bytes
+   available in out buffer, outbuffer should be untouched
+  */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* expect the same result, and not SQL_NO_DATA */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* now provide a space to read the data */
+  outmax= sizeof(SQLWCHAR);
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax, &outlen));
+  is_num(outbuf[0], 0);
+  is_num(outlen, 0);
+
+  /* only now is it unavailable (test with empty and non-empty out buffer) */
+  outmax= 0;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+  outmax= 1; /* outmax greater than 0, but less than sizeof(SQLWCHAR) */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+
+  return OK;
+}
+
+
 /**
  Bug #31246: Opening rowset with extra fields leads to incorrect SQL INSERT
 */
@@ -1909,6 +2028,10 @@ DECLARE_TEST(t_bug31246)
           sizeof(field3), NULL));
   ok_stmt(hstmt, SQLFetch(hstmt));
 
+  is_str(field1, buf, strlen((char *)buf) + 1);
+  is_num(field2, 10);
+  is_str(field3, "Default Text", 13);
+
   /* Clean-up */
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_sql(hstmt, "DROP TABLE t_bug31246");
@@ -1928,6 +2051,7 @@ DECLARE_TEST(t_bug13776)
   SQLULEN     pcColSz;
   SQLCHAR     szColName[MAX_NAME_LEN];
   SQLSMALLINT pfSqlType, pcbScale, pfNullable;
+  SQLLEN display_size, octet_length;
 
   SET_DSN_OPTION(1 << 27);
 
@@ -1935,7 +2059,7 @@ DECLARE_TEST(t_bug13776)
   alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug13776");
-  ok_sql(hstmt1, "CREATE TABLE t_bug13776(ltext LONGTEXT)");
+  ok_sql(hstmt1, "CREATE TABLE t_bug13776(ltext LONGTEXT) CHARSET latin1");
   ok_sql(hstmt1, "INSERT INTO t_bug13776 VALUES ('long text test')");
   ok_sql(hstmt1, "SELECT * FROM t_bug13776");
   ok_stmt(hstmt1, SQLDescribeCol(hstmt1, 1, szColName, MAX_NAME_LEN+1, NULL,
@@ -1944,6 +2068,14 @@ DECLARE_TEST(t_bug13776)
   /* Size of LONGTEXT should have been capped to 1 << 31. */
   is_num(pcColSz, 2147483647L);
 
+  /* also, check display size and octet length (see bug#30890) */
+  ok_stmt(hstmt1, SQLColAttribute(hstmt1, 1, SQL_DESC_DISPLAY_SIZE, NULL,
+                                  0, NULL, &display_size));
+  ok_stmt(hstmt1, SQLColAttribute(hstmt1, 1, SQL_DESC_OCTET_LENGTH, NULL,
+                                  0, NULL, &octet_length));
+  is_num(display_size, 2147483647L);
+  is_num(octet_length, 2147483647L);
+
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug13776");
@@ -1975,9 +2107,15 @@ DECLARE_TEST(t_bug13776_auto)
   SQLCHAR     szFileToLoad[255];
 
   /** @todo get the full path to the library using getenv */
-  env_path= getenv("CommonProgramFiles(x86)");
+#ifdef _WIN64
+  env_path= getenv("CommonProgramW6432");
   if (!env_path)
+  {
     env_path= getenv("CommonProgramFiles");
+  }
+#else
+  env_path= getenv("CommonProgramFiles");
+#endif
 
   if (!env_path)
   {
@@ -2000,7 +2138,7 @@ DECLARE_TEST(t_bug13776_auto)
   alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug13776");
-  ok_sql(hstmt1, "CREATE TABLE t_bug13776(ltext LONGTEXT)");
+  ok_sql(hstmt1, "CREATE TABLE t_bug13776(ltext LONGTEXT) CHARSET latin1");
   ok_sql(hstmt1, "INSERT INTO t_bug13776 VALUES ('long text test')");
   ok_sql(hstmt1, "SELECT * FROM t_bug13776");
   ok_stmt(hstmt1, SQLDescribeCol(hstmt1, 1, szColName, MAX_NAME_LEN+1, NULL,
@@ -2025,6 +2163,144 @@ DECLARE_TEST(t_bug13776_auto)
 }
 
 
+/*
+  Bug#28617 Gibberish when reading utf8 TEXT column through ADO
+*/
+DECLARE_TEST(t_bug28617)
+{
+  SQLWCHAR outbuf[100];
+  SQLLEN outlen;
+
+  ok_sql(hstmt, "select 'qwertyuiop'");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, 0, &outlen),
+              SQL_SUCCESS_WITH_INFO);
+  is_num(outlen, 10);
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, 0, &outlen),
+              SQL_SUCCESS_WITH_INFO);
+  is_num(outlen, 10 * sizeof(SQLWCHAR));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, 100, &outlen));
+
+  is_num(outlen, 10 * sizeof(SQLWCHAR));
+  is(!memcmp(outbuf, W(L"qwertyuiop"), 11 * sizeof(SQLWCHAR)));
+
+  return OK;
+}
+
+
+/*
+  Bug#32684 - chunked retrieval of SQL_C_WCHAR fails
+*/
+DECLARE_TEST(t_bug32684)
+{
+  SQLWCHAR wbuf[20];
+  SQLCHAR abuf[20];
+  SQLLEN wlen, alen;
+  ok_sql(hstmt, "select repeat('x', 100), repeat('y', 100)");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  do
+  {
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, abuf,
+                              20, &alen));
+    printMessage("data= %s, len=%d\n", abuf, alen);
+  } while(alen > 20);
+
+  do
+  {
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, wbuf,
+                              20 * sizeof(SQLWCHAR), &wlen));
+    wprintf(L"# data= %s, len=%d\n\n", wbuf, wlen);
+  } while(wlen > 20 * sizeof(SQLWCHAR));
+
+  return OK;
+}
+
+
+/*
+  Bug #34271 - C/ODBC 5.1 does not list table fields in MSQRY32
+*/
+DECLARE_TEST(t_bug34271)
+{
+  SQLINTEGER x1= 0, x2= 0;
+
+  /* execute the query, but bind only the first column */
+  ok_sql(hstmt, "select 1,2");
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &x1, 0, NULL));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(x1, 1);
+  is_num(x2, 0);
+  x1= 0;
+
+  /* unbind */
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+
+  /* execute the query, but bind only the second column */
+  ok_sql(hstmt, "select 1,2");
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_LONG, &x2, 0, NULL));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(x1, 0);
+  is_num(x2, 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+
+/*
+  Bug #34429 - SQLGetData gives incorrect data
+*/
+DECLARE_TEST(t_bug34429)
+{
+  SQLWCHAR buf[32];
+  SQLLEN reslen;
+
+  ok_sql(hstmt, "drop table if exists t_bug34429");
+  ok_sql(hstmt, "create table t_bug34429 (x varchar(200))");
+  ok_sql(hstmt, "insert into t_bug34429 values "
+                "(concat(repeat('x',32),repeat('y',32),repeat('z',16)))");
+  ok_sql(hstmt, "select x from t_bug34429");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* first chunk */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf),
+                                &reslen), SQL_SUCCESS_WITH_INFO);
+  printMessage("Chunk 1, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 80 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"),
+             32 * sizeof(SQLWCHAR)));
+
+  /* second chunk */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf),
+                                &reslen), SQL_SUCCESS_WITH_INFO);
+  printMessage("Chunk 2, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 49 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"xyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy"),
+             32 * sizeof(SQLWCHAR)));
+
+  /* third chunk */
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf), &reslen));
+  printMessage("Chunk 3, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 18 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"yyzzzzzzzzzzzzzzzz"), 18 * sizeof(SQLWCHAR)));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists t_bug34429");
+
+  return OK;
+}
+
+
 /*
   Bug #32420 - Don't cache results and SQLExtendedFetch work badly together
 */
@@ -2128,8 +2404,7 @@ DECLARE_TEST(t_bug24131)
   SQLLEN boundLen= 0;
   SQLULEN count;
   UWORD status;
-  SQLUINTEGER colSize;
-  SQLRETURN rc;
+  SQLULEN colSize;
 
   ok_sql(hstmt, "drop table if exists bug24131");
 
@@ -2150,13 +2425,8 @@ DECLARE_TEST(t_bug24131)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"show create table bug24131", SQL_NTS));
 
-  rc= SQLDescribeCol(hstmt, 2, NULL, 0, NULL, NULL, &colSize, NULL, NULL);
-
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
-  {
-    printMessage("SQLDescribeCol returned %d", rc);
-    return FAIL;
-  }
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, buff, sizeof(buff), NULL, NULL,
+                                &colSize, NULL, NULL));
 
   ok_stmt(hstmt, SQLBindCol(hstmt,2,SQL_C_BINARY, buff, 1024, &boundLen));
 
@@ -2165,6 +2435,10 @@ DECLARE_TEST(t_bug24131)
 
   ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 1, &count, &status));
 
+  if (sizeof(SQLLEN) == 4)
+    printMessage("colSize: %lu, boundLen: %ld", colSize, boundLen);
+  else
+    printMessage("colSize: %llu, boundLen: %lld", colSize, boundLen);
   is(colSize >= (SQLULEN)boundLen);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -2174,6 +2448,83 @@ DECLARE_TEST(t_bug24131)
 }
 
 
+/*
+  Bug #36069 - SQLProcedures followed by a SQLFreeStmt causes a crash
+ */
+DECLARE_TEST(t_bug36069)
+{
+  SQLSMALLINT size;
+
+  ok_stmt(hstmt, SQLProcedures(hstmt, NULL, 0, NULL, 0,
+                               (SQLCHAR *)"non-existing", SQL_NTS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &size));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"select ?", SQL_NTS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &size));
+
+  return OK;
+}
+
+
+/*
+  Bug #41942 - SQLDescribeCol() segfault with non-zero name length
+  and null buffer
+*/
+DECLARE_TEST(t_bug41942)
+{
+  SQLSMALLINT len;
+  ok_sql(hstmt, "select 1 as name");
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 10, &len,
+                                NULL, NULL, NULL, NULL));
+  is_num(len, 4);
+  return OK;
+}
+
+
+/*
+  Bug 39644 - Binding SQL_C_BIT to an integer column is not working
+ */
+DECLARE_TEST(t_bug39644)
+{
+  char col1 = 0x3f;
+  char col2 = 0xff;
+  char col3 = 0x0;
+  char col4 = 0x1;
+
+  ok_sql(hstmt, "drop table if exists t_bug39644");
+  ok_sql(hstmt, "create table t_bug39644(col1 INT, col2 INT,"\
+	            "col3 BIT, col4 BIT)");
+
+  ok_sql(hstmt, "insert into t_bug39644 VALUES (5, 0, 1, 0)");
+
+  /* Do SELECT */
+  ok_sql(hstmt, "SELECT * from t_bug39644");
+
+  /* Now bind buffers */
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_BIT, &col1, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_BIT, &col2, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 3, SQL_C_BIT, &col3, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 4, SQL_C_BIT, &col4, sizeof(char), 0));
+
+  /* Fetch and check results */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is( col1 == 1 );
+  is( col2 == 0 );
+  is( col3 == 1 );
+  is( col4 == 0 );
+
+  /* Clean-up */
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table t_bug39644");
+
+  return OK;
+}
+
+
 /*
 Bug#32821(it might be duplicate though): Wrong value if bit field is bound to
 other than SQL_C_BIT variable
@@ -2183,13 +2534,10 @@ DECLARE_TEST(t_bug32821)
   SQLRETURN     rc;
   SQLUINTEGER   b;
   SQLUSMALLINT  c;
-  SQLLEN        a_ind, b_ind, c_ind, i;
+  SQLLEN        a_ind, b_ind, c_ind, i, j, k;
   unsigned char a;
 
-#ifdef SQL_NUMERIC_STRUCT_SUPPORTED
   SQL_NUMERIC_STRUCT b_numeric;
-  SQLLEN        j, k
-#endif
 
   SQLUINTEGER par=  sizeof(SQLUSMALLINT)*8+1;
   SQLUINTEGER beoyndShortBit= 1<<(par-1);
@@ -2236,7 +2584,6 @@ DECLARE_TEST(t_bug32821)
     is_num(c, expected_c[i]);
 
     /* Test of binding to numeric - added later so a bit messy */
-#ifdef SQL_NUMERIC_STRUCT_SUPPORTED
     for (k= 1; k < 3; ++k)
     {
       b_ind= sizeof(SQL_NUMERIC_STRUCT);
@@ -2247,16 +2594,15 @@ DECLARE_TEST(t_bug32821)
       {
         b+= (0xff & b_numeric.val[j]) << 8*j;
       }
-      
+
       switch (k)
       {
       case 1: is_num(b, expected_a[i]); break;
       case 2: is_num(b, expected_b[i]); break;
       }
-
+      
     }
-#endif
-
+ 
     ++i;
   }
 
@@ -2267,6 +2613,197 @@ DECLARE_TEST(t_bug32821)
 }
 
 
+/*
+  Bug 55024 - Wrong type returned by SQLColAttribute(SQL_DESC_PRECISION...) in 64-bit Linux
+ */
+DECLARE_TEST(t_bug55024)
+{
+  SQLSMALLINT len;
+  SQLLEN      res;
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "DROP TABLE IF EXISTS t_test55024", SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "CREATE TABLE t_test55024(col01 LONGTEXT, "\
+                                                                  "col02 BINARY(16),"\
+                                                                  "col03 VARBINARY(16),"\
+                                                                  "col04 LONGBLOB,"\
+                                                                  "col05 BIGINT,"\
+                                                                  "col06 TINYINT,"\
+                                                                  "col07 BIT, col08 DOUBLE"\
+                                                                  ") CHARSET latin1", SQL_NTS));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "INSERT INTO t_test55024 VALUES ('a', 'b', 'c', 'd', 999, 111, 1, 3.1415)", SQL_NTS));
+
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "SELECT * FROM t_test55024", SQL_NTS));
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_LONGVARCHAR);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 2, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_BINARY);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 3, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_VARBINARY);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 4, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_LONGVARBINARY);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 5, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_BIGINT);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 6, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_TINYINT);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 7, SQL_DESC_TYPE, NULL, 0, &len, &res));
+  is_num(res, SQL_BIT);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug55024");
+  return OK;
+}
+
+
+/*
+Bug #56677 - SQLNumResultCols() causes the driver to return 
+only first row in the resultset
+*/
+DECLARE_TEST(t_bug56677)
+{
+  SQLINTEGER  nData;
+  SQLCHAR     szData[16];
+  SQLSMALLINT colCount;
+
+  ok_sql(hstmt, "drop table if exists bug56677");
+  ok_sql(hstmt, "CREATE TABLE bug56677 ("\
+    "tt_int INT PRIMARY KEY auto_increment,"\
+    "tt_varchar VARCHAR(128) NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO bug56677 VALUES "\
+    "(100, 'string 1'),"\
+    "(200, 'string 2'),"\
+    "(300, 'string 3'),"\
+    "(400, 'string 4')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, "select * from bug56677", SQL_NTS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &colCount));
+
+  is_num(colCount, 2);
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData),
+    NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(nData, 100);
+  is_str(szData, "string 1", 8);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(nData, 200);
+  is_str(szData, "string 2", 8);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(nData, 300);
+  is_str(szData, "string 3", 8);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(nData, 400);
+  is_str(szData, "string 4", 8);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists bug56677");
+
+  return OK;
+}
+
+
+/* Test of SQLDescribeCol and SQLColAttribute if they are called before SQLExecute.
+   Bug#56717 */
+DECLARE_TEST(t_desccol_before_exec)
+{
+  SQLINTEGER  nData= 200;
+  SQLCHAR     szData[128];
+  SQLSMALLINT colCount;
+  char        colname[MYSQL_NAME_LEN];
+  SQLULEN     collen;
+  SQLLEN      coltype;
+
+  ok_sql(hstmt, "drop table if exists desccol_before_exec");
+  ok_sql(hstmt, "CREATE TABLE desccol_before_exec ("\
+    "tt_int INT PRIMARY KEY auto_increment,"\
+    "tt_varchar VARCHAR(128) CHARACTER SET latin1 NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO desccol_before_exec VALUES "\
+    "(100, 'string 1'),"\
+    "(200, 'string 2'),"\
+    "(300, 'string 3'),"\
+    "(400, 'string 4')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, "select tt_varchar from desccol_before_exec where tt_int > ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, colname, sizeof(colname), NULL,
+    NULL, &collen, NULL, NULL));
+
+  is_str(colname, "tt_varchar", 11);
+  is_num(collen, 128);
+
+  /* Just to make sure that SQLNumResultCols still works fine */
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &colCount));
+
+  is_num(colCount, 1);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER, 0, 0, &nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, szData, sizeof(szData),
+    NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(szData, "string 3", 8);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(szData, "string 4", 8);
+
+  /* Now doing all the same things with SQLColAttribute */
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLPrepare(hstmt, "select tt_int, tt_varchar "
+                                   "from desccol_before_exec "
+                                   "where tt_int <= ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 2, SQL_DESC_TYPE, NULL, 0, NULL, &coltype));
+  is_num(coltype, SQL_VARCHAR);
+
+  /* Just to make sure that SQLNumResultCols still works fine */
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &colCount));
+
+  is_num(colCount, 2);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER, 0, 0, &nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData),
+    NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(szData, "string 1", 8);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(szData, "string 2", 8);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists desccol_before_exec");
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_resultset)
   ADD_TEST(t_convert_type)
@@ -2289,15 +2826,27 @@ BEGIN_TESTS
   ADD_TEST(t_bug16817)
   ADD_TEST(bug6157)
   ADD_TEST(t_binary_collation)
-  ADD_TODO(t_bug29239)
+  ADD_TEST(t_bug29239)
   ADD_TEST(t_bug30958)
+  ADD_TEST(t_bug30958_ansi)
+  ADD_TEST(t_bug30958_wchar)
   ADD_TEST(t_bug31246)
   ADD_TEST(t_bug13776)
   ADD_TEST(t_bug13776_auto)
+  ADD_TEST(t_bug28617)
+  ADD_TEST(t_bug34429)
   ADD_TEST(t_bug32420)
   ADD_TEST(t_bug34575)
   ADD_TEST(t_bug24131)
+  ADD_TEST(t_bug36069)
+  ADD_TEST(t_bug41942)
+  ADD_TEST(t_bug39644)
   ADD_TEST(t_bug32821)
+  ADD_TEST(t_bug34271)
+  ADD_TEST(t_bug32684)
+  ADD_TEST(t_bug55024)
+  ADD_TEST(t_bug56677)
+  ADD_TEST(t_desccol_before_exec)
 END_TESTS
 
 
